#ifndef DATAFOX_LIBRARY_H
#define DATAFOX_LIBRARY_H

// Kommunikationsarray fuer Verbindungen
#include "ChannelBridgeArray.h"
// Versionspruefung
#include "../Shared/Common/DFComUtils.h"

#ifdef _MSC_VER

// GetDllVersion / GetAppVersion
#include <shlwapi.h>

#else

#include </usr/include/pthread.h>
#define WINAPI __stdcall
typedef void *LPVOID;

#endif

// Kontainerklassen fuer Prioritaetswarteschlange importieren.
#include <deque>
#include <queue>

typedef std::deque<CNotifyRecord> notify_deque;
typedef std::priority_queue<CNotifyRecord, notify_deque> notify_priority_deque;

class CPriorityDeque : public notify_priority_deque
{
public:
    // Pruefen ob die Datensatzmeldung des Kanals bereits vorliegt
    int CheckFor(CNotifyRecord &nr)
    {
        int i, res = 0;
        for ( i = 0; i < static_cast<int>( c.size() ); ++i ) 
        {
            if ( nr.channelID == c.at( i ).channelID ) 
            {
                res = 1;
                break;
            }
        }

        return res;
    }

    // Loeschen der Datensatzmeldung des Kanals
    int Erase( int channelID )
    {
        int i, res = 0;
        for ( i = 0; i < static_cast<int>( c.size() ); ++i ) 
        {
            if ( channelID == c.at( i ).channelID ) 
            {
                c.erase( c.begin() + i );
                res = 1;
                break;
            }
        }

        return res;
    }

};

// Diese Klasse wird aus DatafoxLibraryIV.dll exportiert.
class DatafoxLibraryIV
{
public:
	DatafoxLibraryIV();
    virtual ~DatafoxLibraryIV();

    static DatafoxLibraryIV *m_this;

#ifdef _MSC_VER
public:
    HMODULE _hModule;
    int GetVersion( CDFSoftwarebuildVersion &version );
#endif

public:
    int StartNotify();
    int EndNotify();

    // CallBack - Funktion zur Meldung von Datensaetzen.
    // Diese Funktion wird von den Einzelnen CChannel4 Kanalklassen genutzt.
    static int CBNotifyRecord( CNotifyRecord &nr )
    {
#ifdef _MSC_VER
        EnterCriticalSection( &m_this->m_mutexQue );
#else
        pthread_mutex_lock( &m_this->m_mutexQue );
#endif

        // Ist das Melden von Datensaetzen erlaubt?
        if ( !m_this->IsNotifyRecord() ) 
        {
#ifdef _MSC_VER
            LeaveCriticalSection( &m_this->m_mutexQue );
#else
            pthread_mutex_unlock( &m_this->m_mutexQue );
#endif
            return -IV_ERROR_AC_NO_NOTIFY_RECORD;
        }

        // Loeschend oder Hinzuguegen der Datensatzmeldung.
        if ( nr.erase ) 
        {
            // Eintrag des Kanals Entfernen
            if ( m_this->m_queue.Erase( nr.channelID ) ) 
            {
#ifdef _MSC_VER
                //TRACE( "Erase: %i, %i\n", nr.channelID, m_this->m_queue.size() );
                LeaveCriticalSection( &m_this->m_mutexQue );
#else
                pthread_mutex_unlock(&m_this->m_mutexQue);
#endif
                return IV_ERROR_SUCCESS;
            }
        }
        else
        {
            // Bereits Datensatzmeldung des Kanals vorhanden?
            if ( m_this->m_queue.CheckFor( nr ) ) 
            {
                // Verwerfen der Meldung, da bereits/noch verfuegbar.
#ifdef _MSC_VER
                //TRACE( "Skip: %i, %i\n", nr.channelID, m_this->m_queue.size() );
                LeaveCriticalSection( &m_this->m_mutexQue );
#else
                pthread_mutex_unlock( &m_this->m_mutexQue );
#endif
                return IV_ERROR_SUCCESS;
            }

            // Aufnehmen der Datensatzmeldung zu diesem Kanal.
            m_this->m_queue.push( nr );
#ifdef _MSC_VER
            //TRACE( "Put: %i, %i\n", nr.channelID, m_this->m_queue.size() );
#endif
        }

#ifdef _MSC_VER
        LeaveCriticalSection( &m_this->m_mutexQue );
#else
        pthread_mutex_unlock( &m_this->m_mutexQue );
#endif

        return IV_ERROR_SUCCESS;
    }

    // Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:
#ifdef _MSC_VER
    static unsigned long WINAPI ListenThread( LPVOID lpParam );
    static unsigned long WINAPI NotifyThread( LPVOID lpParam );
    CRITICAL_SECTION m_mutexQue;
#else
    static void *ListenThread( void *lpParam );
    static void *NotifyThread( void *lpParam );
    pthread_mutex_t m_mutexQue;
#endif
    volatile int m_endSignal[2];
    volatile int m_initSignal[1];
    enum t_state 
    {
        DOWN, 
        RESET, 
        LISTENING, 
        ACCEPTING
    };
    t_state m_listenThreadState;
#ifdef _MSC_VER
    HANDLE m_workerThread[2];
#else
    pthread_t m_workerThread[2];
#endif
    // Socket fuer Listen der Aktiven Verbindung.
    SOCKET m_listenSocket;
    struct listenThreadData
    {
        char ip[64];
        int port;
        int deviceID;
        int timeout;
        int aliveTO;
        int maxConnections;
        int infoFlags;
    } m_listenThreadData;

    struct notifyThreadData
    {
        char ip[64];
        int port;
        int deviceID;
        int timeout;
        int aliveTO;
        int maxConnections;
        int infoFlags;
    } m_notifyThreadData;

    // Helper
protected:
    /// Anfrage an Kanalarray stellen.
    bool AEChannelRequest( int &channelID );
#ifdef _MSC_VER
    CRITICAL_SECTION m_mutexChannelRequest;
#else
    pthread_mutex_t m_mutexChannelRequest;
#endif

#ifdef WIN32
    //	bool Channel4Request(short &channelID);
#endif

    // Helper
public:
    /// Ermitteln der Kanalklasse aus Array.
    ChannelBridge *GetAEChannel( int &channelID );

    /// Flag ob eine Kanalbindung durchgefuehrt wurde.
    volatile int m_bindSignal;
    /// Ermitteln ob die Kanalklasse an eine bestimmte ID gebunden werden soll.
    int ShouldBoundTo( ChannelBridge *channel );
    /// Kanalobjekt von IDFrom zu IDTo verschieben.
    bool ChannelSwitch( int channelIDFrom, int channelIDTo );
    /// Kanalklasse mit Kanalnummer verbinden.
    bool SetAEChannel( int channelID, ChannelBridge *channel );
    CPriorityDeque m_queue;

    // Abfrageroutine ob ein Kanal Datensaetze gemeldet hat.
    int RecordAvailable( int *channelID, int *deviceID )
    {
        int res = IV_ERROR_SUCCESS;

        // Datensatzmeldung aktiviert?
        if ( !IsNotifyRecord() ) 
            return -IV_ERROR_AC_NO_NOTIFY_RECORD;

        if ( m_queue.size() ) 
        {
#ifdef _MSC_VER
            EnterCriticalSection( &m_mutexQue );
#else
            pthread_mutex_lock( &m_mutexQue );
#endif
            const CNotifyRecord &nr = reinterpret_cast<const CNotifyRecord &>( m_queue.top() );

            *channelID = nr.channelID;
            *deviceID = nr.deviceID;

            m_queue.pop();
#ifdef _MSC_VER
            LeaveCriticalSection( &m_mutexQue );
#else
            pthread_mutex_unlock( &m_mutexQue );
#endif
        } 
        else 
        {
            res = -IV_ERROR_NO_DATA_AVAILABLE;
        }

        return res;
    }

protected:
    int m_notifyRecord; // Datensatzmeldungen verarbeiten (-1 = nicht gesetzt (ja), 0 = nein, 1 = ja)
public:
    int IsNotifyRecord()
    {
        return m_notifyRecord != 0;
    }

public:
    int StartActiveConnection( char *ip, int port, int deviceID, int timeout, int aliveTO, int maxConnections, int infoFlags, int reserved, int *piError );
    int StopActiveConnection();
    int SetRecordAvailable( int enable );
    int SetNotifySoftware( int type, char *ip, int port, int retry, int delay, int state );
    inline bool IsActiveConnection()
    {
        return (m_workerThread[0] != 0);
    }

private:
    unsigned char m_globalCommunicationPassword[16];
    unsigned char m_globalCommunicationPasswordSize;
    int m_globalSessionTimeout;
public: // Passwort fuer die verschluesselte Kommunikation hinterlegen.
    int SetCommunicationPassword( int channelID, unsigned char *password, int size, int sessionTimeout );

    // Membervars
public:
    /// Arrayobjekt fuer Kanalklassen.
    ChannelBridgeArray m_channelArray;

public:
    // Versionspruefungsobjekt
    CDFComUtils m_util;
};

extern DatafoxLibraryIV theApp;

#endif
