/*!
    \file   df_fingerprint.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen der Funktionen für die Arbeit mit Fingertemplates.
*/
#ifndef DF_FINGERPRINT_H
#define DF_FINGERPRINT_H

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010400)

/*!
    \brief Anfügen eines Fingertemplates
           an die Datentabelle des Fingerprints.

    \param connectionId Der zu verwendende Kanal.
    \param deviceAddress Die zu verwendende Busnummer.
    \param templateType Templatetype der übergeben wird.
    \param buffer Zeiger auf die Templatedaten.
    \param bufferSize Die Länge des Buffers auf den der Zeigerfingertemplate verweist.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCFingerprintAppendRecord( int connectionId, int deviceAddress, int templateType, unsigned char *buffer, int bufferSize, int *errorNumber );



/*!
    \brief Lesen eines Fingertemplates

    Über die eindeutige Kombination aus PID/FID können mit dieser Funktion die
    entsprechenden Templatedaten ermittelt werden.

    \param connectionId Der zu verwendende Kanal.
    \param deviceAddress Die zu verwendende Busnummer.
    \param templateType Templatetype der übergeben wird.
    \param pid Personal ID des Templates.
    \param fid Finger ID des Templates.
    \param buffer Zeiger auf die Templatedaten.
    \param bufferSize Als Eingabe die Länge des Buffers auf den der
                 Zeigertemplate verweist, als Ausgabe die übergebene Anzahl Zeichen.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCFingerprintGetRecord( int connectionId, int deviceAddress, int templateType, int pid, int fid, unsigned char *buffer, int *bufferSize, int *errorNumber );



/*!
    \brief Löschen von Fingertemplates

    Sie können einzelne Templates durch Angabe des entsprechenden pid,fid Pärchens löschen.
    Es ist auch möglich durch Angabe einer pid und einer fid = 0, alle Fingertemplates zu dieser pid zu löschen.
    Wird eine pid = 0 und fid = 0 angegeben, werden alle Fingertemplates gelöscht.

    \param connectionId Der zu verwendende Kanal.
    \param deviceAddress Die zu verwendende Busnummer.
    \param pid Personal ID des Templates.
    \param fid Finger ID des Templates.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.04 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCFingerprintDeleteRecord( int connectionId, int deviceAddress, int pid, int fid, int *errorNumber );
#endif

#if (DF_API_VERSION >= 0x04010700)

DF_API_EXTERN int DF_API_CALL DFCFingerprintList( int connectionId, int deviceAddress, int flags, int *errorNumber );

DF_API_EXTERN int DF_API_CALL DFCFingerprintBackup( int connectionId, int deviceAddress, char *filePath, int flags, int *errorNumber );

DF_API_EXTERN int DF_API_CALL DFCFingerprintRestore( int connectionId, int deviceAddress, char *filePath, int flags, int *errorNumber );

#endif
#endif
