#include "../df_fingerprint.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCFingerprintAppendRecord( int connectionId, int deviceAddress, int templateType, unsigned char *buffer, int bufferSize, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, templateType );

    // zulaessige Bereiche pruefen
    if ( !((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || templateType < 0 || templateType > 2
        || (templateType == 0 && bufferSize > 161) 
        || (templateType == 1 && bufferSize > 216)
        || (templateType == 2 && bufferSize > 561)
        || buffer == NULL || errorNumber == NULL || bufferSize < 20 )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t0x%08X\t%i\t%i\t(%i)", buffer, bufferSize, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    // Fingertemplate pruefen (Beide Templatetypen gleichen sich in den ersten 11 Byte)
    StructFingertemplateDIN *fp = (StructFingertemplateDIN*)buffer;
    if ( fp->pidLo != 0 || fp->fid > 10 || fp->quality == 0 || fp->count == 0 )
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd( "\t0x%08X\t%i\t%i\t(%i)", buffer, bufferSize, *errorNumber, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t0x%08X\t%i\t%i\t(%i)", buffer, bufferSize, *errorNumber, 0 );
        return 0;
    }

    // Fingertemplate und size ausgeben.
    // Daten fuer Log in Puffer uebergeben
    unsigned char ucTmp[561 * 3 + 1];
    memset( ucTmp, 0, sizeof(ucTmp) );
    if ( pChannel->logEnabled() )
    {
        int i;
        for ( i = 0; i < bufferSize; ++i )
            sprintf( (char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i] );

        ucTmp[i * 3] = 0;
    }
    else
    {
        sprintf( (char*)ucTmp, "0x%p", buffer );
    }

    pChannel->logAppend( "\t%s\t%i", ucTmp, bufferSize );

    int iRes = pChannel->fingerprintAppendRecord( deviceAddress, templateType, buffer, bufferSize );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 132 )
            {
                // Die Datenbank des Biokeymoduls ist voll.
                *errorNumber = IV_ERROR_MEMORY_FULL;
            }
            else if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    } 
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t(%i)", *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFingerprintGetRecord( int connectionId, int deviceAddress, int templateType, int pid, int fid, unsigned char *buffer, int *bufferSize, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, templateType );

    // zulaessige Bereiche pruefen
    if ( !((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || templateType < 0 || templateType > 2
        || pid == 0 || fid < 1  || fid > 10 || buffer == NULL || bufferSize == NULL || errorNumber == NULL || *bufferSize < 20 )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t%i\t0x%08X\t%i\t%i\t(%i)", pid, fid, buffer, (bufferSize == NULL) ? 0 : *bufferSize, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t%i\t0x%08X\t%i\t%i\t(%i)", buffer, bufferSize, *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->fingerprintGetRecord( deviceAddress, (unsigned char&)templateType, pid, fid, buffer, (int&)*bufferSize );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    // Fingertemplate und size ausgeben.
    // Daten fuer Log in Puffer uebergeben
    unsigned char ucTmp[561 * 3 + 1];
    memset( ucTmp, 0, sizeof(ucTmp) );
    if ( iRes == 1 && pChannel->logEnabled() )
    {
        int i;
        for ( i = 0; i < *bufferSize; ++i )
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);

        ucTmp[i * 3] = 0;
    }
    else
    {
        sprintf( (char*)ucTmp, "0x%p", buffer );
    }

    pChannel->logAppend( "\t%s\t%i", ucTmp, *bufferSize );

    pChannel->logEnd( "\t%i\t(%i)", *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFingerprintDeleteRecord( int connectionId, int deviceAddress, int pid, int fid, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || (pid == 0 && fid != 0) || errorNumber == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t%i\t%i\t(%i)", pid, fid, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t%i\t%i\t(%i)", pid, fid, *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->fingerprintDeleteRecord( deviceAddress, pid, fid );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t%i\t%i\t(%i)", pid, fid, *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFingerprintList( int connectionId, int deviceAddress, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || (flags != 0 && flags != 0x01) || errorNumber == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t%i\t(%i)", flags, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t%i\t(%i)", flags, *errorNumber, 0 );
        return 0;
    }

    int recordCount, iRes = pChannel->fingerprintList( deviceAddress, flags, recordCount );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
        // Anschliessende Ermittlung der Datensatzanzahl erwuenscht?
        if ( flags & 0x01 )
        {
            iRes = recordCount;
        }
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t%i\t(%i)", flags, *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFingerprintBackup( int connectionId, int deviceAddress, char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if (!((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || filePath == 0 || flags < 0 || flags > 1 || errorNumber == NULL ) 
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", (filePath == NULL) ? "" : filePath, flags, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->fingerprintBackup( deviceAddress, filePath, flags );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCFingerprintRestore( int connectionId, int deviceAddress, char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( !((deviceAddress >= 1 && deviceAddress <= 250) || deviceAddress == 254)
        || filePath == 0 || flags < 0 || flags > 1 || errorNumber == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", (filePath == NULL) ? "" : filePath, flags, IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() )
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->fingerprintRestore( deviceAddress, filePath, flags );
    if ( iRes == IV_ERROR_SUCCESS )
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if ( iRes > IV_ERROR_SUCCESS )
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ( (iRes & 0xFF) == 132 )
            {
                // Die Datenbank des Biokeymoduls ist voll.
                *errorNumber = IV_ERROR_MEMORY_FULL;
            }
            else if ( (iRes & 0xFF) == 28 )
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ( (iRes & 0xFF) == 29 )
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
            /*
            switch(iRes & 0xFF) {
            case 0:
            break;
            default:
            }*/
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%s\t%i\t%i\t(%i)", filePath, flags, *errorNumber, iRes );

    return iRes;
}
