#ifndef DEVICEINFO_H
#define DEVICEINFO_H

#include "hardwareinfo.h"
#include "softwareinfo.h"
#include "firmwareoptions.h"

class DeviceInfo
{
public:
    DeviceInfo();
    virtual ~DeviceInfo();

public:
    void clear();

    void setVersionBootloader( const std::string &version );
    std::string versionBootloader() const;

    void setBootloaderMode( bool active );
    bool bootloaderMode() const;

    void setHardwareInfo( const HardwareInfo &hardwareInfo );
    HardwareInfo &hardwareInfo();
    
    void setFileHardwareInfo( const FileHardwareInfo &file );
    FileHardwareInfo &fileHardwareInfo();
    setItemBoard collectModuleIds( const setModuleId &ids );

    void setVersionFirmware( const std::string &version );
    std::string versionFirmware() const;

    void setSoftwareInfo( const SoftwareInfo &softwareInfo );
    SoftwareInfo &softwareInfo();

    void setFirmwareOptions( const FirmwareOptions &firmwareOptions );
    FirmwareOptions &firmwareOptions();

    void setSerialNumber( unsigned int serial );
    unsigned int serialNumber() const;

    void setPasswordKey( const std::string &key );
    std::string passwordKey() const;

    void setVersionFingerprint( const std::string &version );
    std::string versionFingerprint() const;

    void setVersionUSBHost( const std::string &version );
    std::string versionUSBHost() const;

    void setVersionGPS( const std::string &version );
    std::string versionGPS() const;

    void setMacAddress( const std::string &address );
    std::string macAddress() const;

    void setAccessControlMasterId( const std::string &id );
    std::string accessControlMasterId() const;

    void setTimeboyGroupId( const std::string &id );
    std::string timeboyGroupId() const;

public: // Helper / Shorties
    bool importStructHardwareInfo( const StructHardwareInfo &info );
    StructHardwareInfo structHardwareInfo() const;

    void setStructSoftwareInfo( const StructSoftwareInfo &info );
    StructSoftwareInfo structSoftwareInfo() const;

    StructSoftwareInfo checkInfo( const StructSoftwareInfo &info );
    int SoftwareInfoToString( const StructSoftwareInfo &swInfo, std::string &info );

    int compareInfos( const StructHardwareInfo &hwInfo, const StructSoftwareInfo &swInfo );

public: // Shorties fr FirmwareOptions
    void setSystemFlags( unsigned int flags );
    unsigned int systemFlags() const;

    void setTypeExternalReader( unsigned char type );
    unsigned char getTypeExternalReader();

    void setOptionIOModule( unsigned char enabled );
    unsigned char getOptionIOModule();

    void setOptionTBAdaption( unsigned char enabled );
    unsigned char getOptionTBAdaption();

    void setOptionComServer( unsigned char enabled );
    unsigned char getOptionComServer();

    void setOptionTPWrite( unsigned char enabled );
    unsigned char getOptionTPWrite();

    void setOptionHTTP( unsigned char enabled );
    unsigned char getOptionHTTP();

    void setOptionLanguage( unsigned char enabled );
    unsigned char getOptionLanguage();

    void setOptionBiokeyUpdate( unsigned char enabled );
    unsigned char getOptionBiokeyUpdate();

    void setSetupOptions( unsigned short mask, unsigned short options );
    unsigned short getSetupOptions( unsigned short mask );

    void setPayableOptions( unsigned char mask, unsigned char options );
    unsigned char getPayableOptions( unsigned char mask );

public: // Fehlerbehandlung
    std::string m_errorText;

private: // Variablen
    std::string _versionBootloader;
    bool _bootloaderMode;
    std::string _versionFirmware;

    unsigned int _serialNumber;

    HardwareInfo _hardwareInfo;

    std::string _versionFingerprint;
    std::string _versionUSBHost;
    std::string _versionGPS;

    std::string _macAddress;

    SoftwareInfo _softwareInfo;
    std::string _passwordKey;

    FirmwareOptions _firmwareOptions;

    std::string _accessControlMasterId;
    std::string _timeboyGroupId;
};

#endif
