/**
 * @file  FileIni.h
 * @brief Schnittstelle fuer die Klasse FileIni.
 */
//////////////////////////////////////////////////////////////////////
#ifndef FILEINI_H
#define FILEINI_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

/* Verwaltung einer Initialisierungsdatei mit Sektionen [...] und 
 * Schluesseln mit Werten vom Typ "Zahl" und "Zeichenkette".
 *
 *
 */
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _MSC_VER
#define stat _stat
#endif

// Maximale Laenge von Sektionsnamen
#define MAX_SECTION_NAME   64
// Maximale Laenge von Schluesselnamen
#define MAX_KEY_NAME       64
// Maximale Laenge von Aufnehmbaren Werten
#define MAX_VALUE_NAME     258
// Maximale Laenge von Aufnehmbaren Kommentarzeilen
#define MAX_COMMEND_NAME   258

class FileIni
{
public:
    FileIni();
    virtual ~FileIni();

    // Pruefen ob die Datei vorhanden und grundlegend in Ordnung
    int fileExists(const char *filePath);

    // Lesen eines vorzeichenlosen Integerwertes
    unsigned int GetUInt(const char *section, const char *key, int defaultVal, const char *fileName);
    // Lesen einer Zeichenkette
    int GetString(const char *section, const char *key, char *val, const char *fileName);
    int SetString(const char *section, const char *key, const char *val, const char *fileName);

protected:
// FindFlag
#define FF_CREATE 0x00000001 // Erzeugen wenn nicht vorhanden

    int FindSection(FILE *fp, const char *section, unsigned int flags);
    int FindKey(FILE *fp, const char *key, unsigned int flags, char *line, int length);
    unsigned int GetNum(int defaultVal, char *line);
    int GetStr(char *val, char *line);
    //unsigned int SetNum(FILE *fp, int iDefault, char *szLine);
    int SetStr(FILE *fp, const char *val, char *line);

    // Druckbares Zeichen, auch Umlaute
    int IsPrint(char c) // ISO 8859-1
    {
        unsigned char uc = static_cast<unsigned char>( c );
        // ä=0xE4, ö=0xF6, ü=0xFC, Ä=0xC4, Ö=0xD6, Ü=0xDC, ß=0xDF
        return (isprint(c) || uc == 0xE4 || uc == 0xF6 || uc == 0xFC
            || uc == 0xDF || uc == 0xC4 || uc == 0xD6 || uc == 0xDC);
    }

public:
// ScanFlag
#define SF_READ   0x00000001 // Werte lesen
#define SF_WRITE  0x00000002 // Werte schreiben
#define SF_CREATE 0x00000010 // Sektion sowie Schluessel erzeugen wenn noch nicht vorhanden
#define SF_DELETE 0x00000020 // Sektion sowie Schluessel loeschen
// der value - Paramter kann auf folgende Werte verweisen
#define SF_VALUE_COMMEND 0x00000100 // Wert ist eine Kommentarzeile
#define SF_VALUE_STRING  0x00000200 // Wert ist eine Zeichenkette
#define SF_VALUE_DIGIT   0x00000400 // Wert ist eine Zahl

    int Scan(const char *section, const char *key, char *val, unsigned int flags, const char *fileName);
};

#endif

