/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "Channel4ModuleSystem.h"

#include <memory>
#include <stdlib.h>
#include <ctype.h>
#include <sstream>
#include <string>
#include <boost/scoped_array.hpp>
#include <boost/algorithm/string.hpp>
#include <regex>

#include "../Common/DFSoftwareVersion.h"

void Channel4ModuleSystem::mobileMaskedSysvar( char *buf )
{
    int i;
    char lBuf[IV_PACKET_LENGTH_DATA];
    char c1, c2, c3, c4;
    c1 = c2 = c3 = c4 = '\0';
    memset( lBuf, 0, sizeof(lBuf) );

    int lenBuf = (int)strlen( buf );
    for ( i = 0; i < lenBuf; ++i )
    {
        switch ( (unsigned char)buf[i] )
        {
        case 0x40: strcat(lBuf, "\\00"); break;
        case 0xA3: strcat(lBuf, "\\01"); break;
        case 0x24: strcat(lBuf, "\\02"); break;
        case 0xA5: strcat(lBuf, "\\03"); break;
        case 0xE8: strcat(lBuf, "\\04"); break;
        case 0xE9: strcat(lBuf, "\\05"); break;
        case 0xF9: strcat(lBuf, "\\06"); break;
        case 0xEC: strcat(lBuf, "\\07"); break;
        case 0xF2: strcat(lBuf, "\\08"); break;
        case 0xC7: strcat(lBuf, "\\09"); break;
        case 0xD8: strcat(lBuf, "\\0B"); break;
        case 0xF8: strcat(lBuf, "\\0C"); break;
        case 0xC5: strcat(lBuf, "\\0E"); break;
        case 0xE5: strcat(lBuf, "\\0F"); break;
        case 0x5F: strcat(lBuf, "\\11"); break;
        case 0x0C: strcat(lBuf, "\\1B0A"); break;
        case 0x5E: strcat(lBuf, "\\1B14"); break;
        case 0x7B: strcat(lBuf, "\\1B28"); break;
        case 0x7D: strcat(lBuf, "\\1B29"); break;
        case 0x5C: {
            size_t len = lenBuf - i;
            if ( len > 2 )
            {
                c1 = (char)toupper( buf[i + 1] );
                c2 = (char)toupper( buf[i + 2] );
                if  ( len > 4 )
                {
                    c3 = (char)toupper( buf[i + 3] );
                    c4 = (char)toupper( buf[i + 4] );
                }
                // Handelt es sich um eine bekannte Maskierung?
                // Dann wird diese mit übernommen,
                // ansonsten wird der \ maskiert.
                if ( c1 == '0' )
                {
                    if ( c2 == '0' || c2 == '1' || c2 == '2' || c2 == '3'
                        || c2 == '4' || c2 == '5' || c2 == '6' || c2 == '7'
                        || c2 == '8' || c2 == '9' || c2 == 'B' || c2 == 'C'
                        || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        continue;
                    }
                }
                else if ( (len > 4) && c1 == '1' && c2 == 'B' )
                {
                    if ((c3 == '0' && c4 == 'A')
                        || (c3 == '1' && c4 == '4')
                        || (c3 == '2' && c4 == '8')
                        || (c3 == '2' && c4 == '9')
                        || (c3 == '2' && c4 == 'F')
                        || (c3 == '3' && c4 == 'C')
                        || (c3 == '3' && c4 == 'D')
                        || (c3 == '3' && c4 == 'E')
                        || (c3 == '4' && c4 == '0') )
                    {
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        continue;
                    }

                }
                else if ( c1 == '1' )
                {
                    if ( c2 == '1' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        continue;
                    }
                }
                else if ( (c1 == '2' && c2 == '4')
                    || (c1 == '4' && c2 == '0')
                    || (c1 == '6' && c2 == '0') )
                {
                    lBuf[strlen(lBuf)] = buf[i]; i++;
                    lBuf[strlen(lBuf)] = buf[i]; i++;
                    lBuf[strlen(lBuf)] = buf[i];
                    continue;
                }
                else if ( c1 == '5' )
                {
                    if ( c2 == 'B' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        continue;
                    }
                }
                else if ( c1 == '7' )
                {
                    if ( c2 == 'B' || c2 == 'C' || c2 == 'D' || c2 == 'E' || c2 == 'F' )
                    {
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        i++;
                        lBuf[strlen(lBuf)] = buf[i];
                        continue;
                    }
                }
            }
            strcat( lBuf, "\\1B2F" ); 
            break;}
        case 0x5B: strcat( lBuf, "\\1B3C" ); break;
        case 0x7E: strcat( lBuf, "\\1B3D" ); break;
        case 0x5D: strcat( lBuf, "\\1B3E" ); break;
        case 0x7C: strcat( lBuf, "\\1B40" ); break;
        case 0xC6: strcat( lBuf, "\\1C" ); break;
        case 0xE6: strcat( lBuf, "\\1D" ); break;
        case 0xDF: strcat( lBuf, "\\1E" ); break;
        case 0xC9: strcat( lBuf, "\\1F" ); break;
        case 0xA4: strcat( lBuf, "\\24" ); break;
        case 0xA1: strcat( lBuf, "\\40" ); break;
        case 0xC4: strcat( lBuf, "\\5B" ); break;
        case 0xD6: strcat( lBuf, "\\5C" ); break;
        case 0xD1: strcat( lBuf, "\\5D" ); break;
        case 0xDC: strcat( lBuf, "\\5E" ); break;
        case 0xA7: strcat( lBuf, "\\5F" ); break;
        case 0xBF: strcat( lBuf, "\\60" ); break;
        case 0xE4: strcat( lBuf, "\\7B" ); break;
        case 0xF6: strcat( lBuf, "\\7C" ); break;
        case 0xF1: strcat( lBuf, "\\7D" ); break;
        case 0xFC: strcat( lBuf, "\\7E" ); break;
        case 0xE0: strcat( lBuf, "\\7F" ); break;
        default: lBuf[strlen(lBuf)] = buf[i];
        }
    }

    // Ggf. geänderten Wert übernehmen
    strcpy( buf, lBuf );
}

void Channel4ModuleSystem::mobileUnmaskedSysvar( char *buf )
{
    int i;
    char c1, c2, c3, c4;
    c1 = c2 = c3 = c4 = '\0';
    char lBuf[IV_PACKET_LENGTH_DATA];
    memset( lBuf, 0, sizeof(lBuf) );

    int len = (int)strlen( buf );
    for ( i = 0; i < len; ++i )
    {
        if ( ((len - i) > 2) && buf[i] == '\\' ) 
        {
            c1 = (char)toupper( buf[i + 1] );
            c2 = (char)toupper( buf[i + 2] );
            if ( (len - i) > 4 )
            {
                c3 = (char)toupper( buf[i + 3] );
                c4 = (char)toupper( buf[i + 4] );
            }

            if( c1 == '0' )
            {
                if (c2 == '0') { lBuf[strlen(lBuf)] = (char)0x40; i += 2; }
                else if (c2 == '1') {lBuf[strlen(lBuf)] = (char)0xA3; i += 2; }
                else if (c2 == '2') { lBuf[strlen(lBuf)] = (char)0x24; i += 2; }
                else if (c2 == '3') { lBuf[strlen(lBuf)] = (char)0xA5; i += 2; }
                else if (c2 == '4') { lBuf[strlen(lBuf)] = (char)0xE8; i += 2; }
                else if (c2 == '5') { lBuf[strlen(lBuf)] = (char)0xE9; i += 2; }
                else if (c2 == '6') { lBuf[strlen(lBuf)] = (char)0xF9; i += 2; }
                else if (c2 == '7') { lBuf[strlen(lBuf)] = (char)0xEC; i += 2; }
                else if (c2 == '8') { lBuf[strlen(lBuf)] = (char)0xF2; i += 2; }
                else if (c2 == '9') { lBuf[strlen(lBuf)] = (char)0xC7; i += 2; }
                else if (c2 == 'B') { lBuf[strlen(lBuf)] = (char)0xD8; i += 2; }
                else if (c2 == 'C') { lBuf[strlen(lBuf)] = (char)0xF8; i += 2; }
                else if (c2 == 'E') { lBuf[strlen(lBuf)] = (char)0xC5; i += 2; }
                else if (c2 == 'F') { lBuf[strlen(lBuf)] = (char)0xE5; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if ( c1 == '1' )
            {
                if ( c2 == '1')
                { 
                    lBuf[strlen(lBuf)] = (char)0x5F;
                    i += 2;
                }
                else if ( c2 == 'B' )
                {
                    if (c3 == '0' && c4 == 'A') { lBuf[strlen(lBuf)] = (char)0x0C; i += 4; }
                    else if (c3 == '1' && c4 == '4') { lBuf[strlen(lBuf)] = (char)0x5E; i += 4; }
                    else if (c3 == '2' && c4 == '8') { lBuf[strlen(lBuf)] = (char)0x7B; i += 4; }
                    else if (c3 == '2' && c4 == '9') { lBuf[strlen(lBuf)] = (char)0x7D; i += 4; }
                    else if (c3 == '2' && c4 == 'F') { lBuf[strlen(lBuf)] = (char)0x5C; i += 4; }
                    else if (c3 == '3' && c4 == 'C') { lBuf[strlen(lBuf)] = (char)0x5B; i += 4; }
                    else if (c3 == '3' && c4 == 'D') { lBuf[strlen(lBuf)] = (char)0x7E; i += 4; }
                    else if (c3 == '3' && c4 == 'E') { lBuf[strlen(lBuf)] = (char)0x5D; i += 4; }
                    else if (c3 == '4' && c4 == '0') { lBuf[strlen(lBuf)] = (char)0x7C; i += 4; }
                    else { lBuf[strlen(lBuf)] = buf[i]; }
                }
                else if (c2 == 'C') { lBuf[strlen(lBuf)] = (char)0xC6; i += 2; }
                else if (c2 == 'D') { lBuf[strlen(lBuf)] = (char)0xE6; i += 2; }
                else if (c2 == 'E') { lBuf[strlen(lBuf)] = (char)0xDF; i += 2; }
                else if (c2 == 'F') { lBuf[strlen(lBuf)] = (char)0xC9; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if ( c1 == '2' )
            {
                if (c2 == '4') { lBuf[strlen(lBuf)] = (char)0xA4; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if ( c1 == '4' )
            {
                if ( c2 == '0' ) { lBuf[strlen(lBuf)] = (char)0xA1; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if ( c1 == '5' )
            {
                if (c2 == 'B') { lBuf[strlen(lBuf)] = (char)0xC4; i += 2; }
                else if (c2 == 'C') { lBuf[strlen(lBuf)] = (char)0xD6; i += 2; }
                else if (c2 == 'D') { lBuf[strlen(lBuf)] = (char)0xD1; i += 2; }
                else if (c2 == 'E') { lBuf[strlen(lBuf)] = (char)0xDC; i += 2; }
                else if (c2 == 'F') { lBuf[strlen(lBuf)] = (char)0xA7; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if (c1 == '6')
            {
                if (c2 == '0') { lBuf[strlen(lBuf)] = (char)0xBF; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            }
            else if (c1 == '7')
            {
                if (c2 == 'B') { lBuf[strlen(lBuf)] = (char)0xE4; i += 2; }
                else if (c2 == 'C') { lBuf[strlen(lBuf)] = (char)0xF6; i += 2; }
                else if (c2 == 'D') { lBuf[strlen(lBuf)] = (char)0xF1; i += 2; }
                else if (c2 == 'E') { lBuf[strlen(lBuf)] = (char)0xFC; i += 2; }
                else if (c2 == 'F') { lBuf[strlen(lBuf)] = (char)0xE0; i += 2; }
                else { lBuf[strlen(lBuf)] = buf[i]; }
            } else {

                lBuf[strlen(lBuf)] = buf[i];
            }
        } 
        else 
        {
            lBuf[strlen(lBuf)] = buf[i];
        }
    }

    // Ggf. geänderten Wert übernehmen
    strcpy( buf, lBuf );
}


// ##################################################################
/**
* @brief Initialisieren der Membervariablen.
*
* @param id Nummer der Kanalklasse fuer z.B. Trace.
*/
Channel4ModuleSystem::Channel4ModuleSystem( short channelId, short deviceId ) : Channel4ModuleHardware( channelId, deviceId ) 
{
    // Seriennummer des Geraetes
    _serial = 0xFFFFFFFF;

    // Passwort fuer die Kommunikation
    memset( _passwordCommunication, 0, sizeof(_passwordCommunication) );
    _isPasswordProtected = 0;
    _isPasswordProtection = 0;

    // Firmwareversion des Geraetes.
    memset( _versionFirmware, 0, sizeof(_versionFirmware) );
    
    // Transparentmodus
    _transparencyMode = Off;

    // Streamtransfer
    m_streamBuffer.clear();
    m_streamTransfer = false;
}

// ##################################################################
/**
* @brief Destruktor.
*
*/
Channel4ModuleSystem::~Channel4ModuleSystem() 
{
}

// ##################################################################
/**
 * @brief Lesen aus der Kommunikationsschnittstelle.
 */
int Channel4ModuleSystem::readData( void *buffer, int readCount, int &readInCount, int readTrials )
{
    trace( "%s(%i):INFO(3): %s( %p, %i, %i, %i )\n", __FILE__, __LINE__, __func__, buffer, readCount, readInCount, readTrials );

    int res;
    switch ( _transparencyMode )
    {
        case Off:
        case Raw:
            res = Channel4Base::readData( buffer, readCount, readInCount, readTrials );
            break;

        case Masked:
        {
            char dataByte;
            char *data = reinterpret_cast<char *>( buffer );
            int dataByteCount;
            bool masked = false;
            int maskedCount = 0;
#ifdef _DEBUG
            clock_t clockStart = clock();
#endif
            // Datenstrom zeichenweise bearbeiten und besondere Zeichen maskieren
            for ( readInCount = 0; readInCount < readCount; ++readInCount )
            {	
                if ( (res = Channel4Base::readData( &dataByte, 1, dataByteCount, readTrials )) != IV_ERROR_SUCCESS )
                {
#ifdef _DEBUG
                    std::string prefix = boost::str( boost::format( "(%4.3f)\n" ) % (static_cast<double>(clock() - clockStart) / CLOCKS_PER_SEC) );
                    trace( prefix.c_str() );
#endif
                    return res;
                }
#ifdef _DEBUG
                if ( readInCount == (readCount - 1) && dataByte != IV_PACKET_SIGN_ESCAPE )
                    traceData( reinterpret_cast<unsigned char *>( &dataByte ), 1, TraceDataEnd );
                else if ( readInCount == 0 )
                    traceData( reinterpret_cast<unsigned char *>( &dataByte ), 1, TraceDataBegin );
                else
                    traceData( reinterpret_cast<unsigned char *>( &dataByte ), 1, TraceDataAppend ); 
#endif
                if ( dataByte == '~' )
                {
                    _transparencyMode = Off;
                }

                if ( dataByte == IV_PACKET_SIGN_ESCAPE )
                {
                    masked = true;
                    readInCount--;
                } 
                else if ( masked )
                {
                    if ( dataByte == 0x00 )
                        data[readInCount] = IV_PACKET_SIGN_ESCAPE;
                    else if ( dataByte == 0x01 )
                        data[readInCount] = IV_PACKET_SIGN_START;
                    else
                        data[readInCount] = dataByte;

                    maskedCount++;
                    masked = false;
                }
                else
                {
                    data[readInCount] = dataByte;
                }
            }
#ifdef _DEBUG
            std::string prefix = boost::str( boost::format( "(%4.3f)\n" ) % (static_cast<double>(clock() - clockStart) / CLOCKS_PER_SEC) );
            trace( prefix.c_str() );
#endif
            break;
        }

        default:
            trace( "%s(%i):ERROR(1): %s() [ Unbekannter Mode (%i) ]\n", __FILE__, __LINE__, __func__, _transparencyMode );
            return -IV_ERROR_TRANSMIT;
    }

    return res;
}

// ##################################################################
/**
 * @brief Schreiben in die Kommunikationsschnittstelle.
 */
int Channel4ModuleSystem::writeData( const void *buffer, int writeCount, int &writeOutCount, int writeTrials )
{
    trace( "%s(%i):INFO(3): %s( %p, %i, %i, %i )\n", __FILE__, __LINE__, __func__, buffer, writeCount, writeOutCount, writeTrials );

    int res;
    switch ( _transparencyMode )
    {
        case Off:
        case Raw:
            res = Channel4Base::writeData( buffer, writeCount, writeOutCount, writeTrials );
            break;

        case Masked:
        {
            // Für das Versenden als gesamten Block, die Daten maskiert in einen stringstream packen.
            std::vector<unsigned char> v;
            const unsigned char *data = reinterpret_cast<const unsigned char *>( buffer );
            int masked = 0;
#ifdef _DEBUG
            clock_t clockStart = clock();
#endif
            // Datenstrom zeichenweise bearbeiten und besondere Zeichen maskieren
            for ( int count = 0; count < writeCount; ++count )
            {	
                // Paketzeichen im Datenstrom mit Escapesequenz umschreiben
                if ( (data[count] == IV_PACKET_SIGN_START) || (data[count] == IV_PACKET_SIGN_ESCAPE) )
                {
                    v.push_back( IV_PACKET_SIGN_ESCAPE );
                    // Anzahl maskierte mitzaehlen
                    masked++;

                    switch ( data[count] ) 
                    {
                    case IV_PACKET_SIGN_START:
                        v.push_back( 0x01 );
                        break;

                    case IV_PACKET_SIGN_ESCAPE:
                        v.push_back( 0x00 );
                        break;
                    }
                } 
                else 
                {
                    v.push_back( data[count] );
                }
            }
#ifdef _DEBUG  
            std::string prefix = boost::str( boost::format( "(%4.3f) write:" ) % (static_cast<double>(clock() - clockStart) / CLOCKS_PER_SEC) );
            traceData( reinterpret_cast<unsigned char *>( v.data() ), v.size(), TraceDataFullWithCrLf, prefix.c_str() );
#endif
            res = Channel4Base::writeData( v.data(), v.size(), writeOutCount, writeTrials );
            writeOutCount -= masked; // Die Maskierungszeichen zählen nicht als mitgesendet.
            break;
        }

        default:
            trace( "%s(%i):ERROR(1): %s() [ Unbekannter Mode (%i) ]\n", __FILE__, __LINE__, __func__, _transparencyMode );
            return -IV_ERROR_TRANSMIT;
    }

    return res;
}

#include <boost/format.hpp>
#include <iomanip>
// ##################################################################
/**
* @brief Remoteaufruf einer lokalen Modulfunktion.
*/
void Channel4ModuleSystem::traceData( unsigned char *data, int size, EnumTraceDataMode mode, const char *prefix )
{
    if ( size > 0 )
    {
        // Byte-Array in Hex-String wandeln und ausgeben.
        std::string buf;

        for ( int len = 0; len < size; ++len )
        {
            if ( len == 0 && (mode == TraceDataFull || mode == TraceDataFullWithCrLf || mode == TraceDataBegin) )
            {
                // Prefix zuerst ausgeben?
                if ( prefix != nullptr )
                    buf = prefix;
                
                buf += boost::str( boost::format( "[%1$02X " ) % static_cast<int>( data[len] ) );
            }
            else if ( len == (size - 1) && (mode == TraceDataFull || mode == TraceDataFullWithCrLf || mode == TraceDataEnd || mode == TraceDataEndWithCrLf) )
            {
                buf += boost::str( boost::format( "%1$02X]" ) % static_cast<int>( data[len] ) );
                // CrLf anfügen?
                if ( mode == TraceDataFullWithCrLf || mode == TraceDataEndWithCrLf )
                    buf += "\n";                
            }
            else
            {
                buf += boost::str( boost::format( "%1$02X " ) % static_cast<int>( data[len] ) );
            }
        }

        trace( buf.c_str() );
    }
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Channel4ModuleSystem::getErrorModule( short module, short error, std::string &text )
{
    trace("%s(%i):INFO(3): %s( %i, %i, %p )\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_SYSTEM ?
    if ( module == IV_MODULE_SYSTEM )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleHardware::getErrorModule( module, error, text );
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Channel4ModuleSystem::getErrorFunction( short module, short func, short error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %i, %p )\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_SYSTEM )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 ) 
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %i ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %i ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_SYSTEM_RESET:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Resetmodus unbekannt.") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_SERIAL:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Pruefung der Seriennummer - Verschluesselung schlug fehl.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_GET_VERSION:
        case IV_FUNCTION_GET_VERSION_BOOTLOADER:
        case IV_FUNCTION_GET_SERIAL:
        case IV_FUNCTION_SET_RTC:
        case IV_FUNCTION_GET_RTC:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_SYSTEM_VARIABLE:
        case IV_FUNCTION_GET_SYSTEM_VARIABLE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Der zu setzende Variablenwert ist zu lang.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_RELAY:
        case IV_FUNCTION_GET_RELAY:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Bereichsname unbekannt. (INTERN, TMR, IOM ...)") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %i ): Busnummer (z.B. TMR) nicht bekannt oder erreichbar.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Relaisnummer unbekannt.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %i ): Zu setzender Zustand (Status) unbekannt oder nicht unterstützt.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_SET_MMC:
        case IV_FUNCTION_SET_FLASH:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Speicherseite konnte nicht geschrieben werden.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Speicherplatz reicht nicht aus.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_GET_SFSMPB:
        case IV_FUNCTION_GET_MMC:
        case IV_FUNCTION_GET_FLASH:
        case IV_FUNCTION_SET_EEPROM:
        case IV_FUNCTION_GET_EEPROM:
        case IV_FUNCTION_SET_PROGRAM:
        case IV_FUNCTION_GET_PROGRAM:
        case IV_FUNCTION_GET_RAM:
        case IV_FUNCTION_SET_RAM:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_SET_SEGMENT:
        case IV_FUNCTION_END_SET_SEGMENT:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Der Typ ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %i ): Die Updatesegmente sind erschöpft.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %i ): Es wurde kein STOP vorm START durchgeführt.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %i ): Verschachtelungsfehler von STOP und START.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %i ): Prüfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener überein.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_SET_SEGMENT_OK:
        case IV_FUNCTION_END_SET_SEGMENT_OK:
            break;

        case IV_FUNCTION_BEGIN_GET_SEGMENT:
        case IV_FUNCTION_END_GET_SEGMENT:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Der Typ ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %i ): Die Updatesegmente sind erschöpft.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Der Benötigte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %i ): Es wurde kein STOP vorm START durchgeführt.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %i ): Verschachtelungsfehler von STOP und START.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %i ): Prüfsumme (CRC) des Updatesegmentes stimmt nicht mit gegebener überein.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_BEGIN_GET_SEGMENT_OK:
        case IV_FUNCTION_END_GET_SEGMENT_OK:
            break;

        case IV_FUNCTION_ENTER_BIOS:
        case IV_FUNCTION_EXIT_BIOS:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): ") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_ENTERED_BIOS:
        case IV_FUNCTION_EXITED_BIOS:
            break;

        case IV_FUNCTION_GET_INFO:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Der Majorbestandteil des Modulnamens ist unbekannt.") % error );
                break;

            case 2:
                text = boost::str( boost::format( "( %i ): Der Minorbestandteil des Modulnamens ist unbekannt.") % error );
                break;

            case 3:
                text = boost::str( boost::format( "( %i ): Es wurden zu wenige Paramteter angegeben.") % error );
                break;

            case 4:
                text = boost::str( boost::format( "( %i ): Es wurden zu viele Paramteter angegeben.") % error );
                break;

            case 5:
                text = boost::str( boost::format( "( %i ): Der Wert eines Parameters liegt auserhalb seines Gültigkeitsbereichs.") % error );
                break;

            case 6:
                text = boost::str( boost::format( "( %i ): Die Information zur Komponente steht nicht zur Verfügung.") % error );
                break;

            case 7:
                text = boost::str( boost::format( "( %i ): Das Modul kann zur Zeit nicht angesprochen werden.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_OPEN_COM_SERVER_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Es wurden zu wenige Paramteter angegeben.") % error );
                break;
            case 2:
                text = boost::str( boost::format( "( %i ): Es wurden zu viele Paramteter angegeben.") % error );
                break;
            case 3:
                text = boost::str( boost::format( "( %i ): Die Angabe des comm Parameters wird nicht unterstützt.") % error );
                break;
            case 4:
                text = boost::str( boost::format( "( %i ): Die Angabe des baudRate Parameters wird nicht unterstützt.") % error );
                break;
            case 5:
                text = boost::str( boost::format( "( %i ): Die Angabe des byteSize Parameters wird nicht unterstützt.") % error );
                break;
            case 6:
                text = boost::str( boost::format( "( %i ): Die Angabe des parity Paramters wird nicht unterstützt.") % error );
                break;
            case 7:
                text = boost::str( boost::format( "( %i ): Die Angabe des stopBits Paramters wird nicht unterstützt.") % error );
                break;
            case 8:
                text = boost::str( boost::format( "( %i ): Die Angabe des flags Parameters wird nicht unterstützt.") % error );
                break;
            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_CLOSE_COM_SERVER_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Bei der Übertragung ist ein Overflow eingetreten.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_OPEN_COM_SERVER_MODE_OK:
            break;

        case IV_FUNCTION_PASSWORD_CHECK:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Passwort stimmt nicht mit dem benötigten überein.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_TRANSMIT_STREAM:
            switch ( error )
            {
            case 3:
                text = boost::str( boost::format( "( %i ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        case IV_FUNCTION_TRANSPARENCY_MODE:
            switch ( error )
            {
            case 1:
                text = boost::str( boost::format( "( %i ): Der Angeforderte Speicherplatz steht nicht zur Verfügung.") % error );
                break;

            case 2: // Kann kommen wenn das Flag-Bit3 sitzt. Es wird damit ein TimeboyI-III gefordert.
                text = boost::str( boost::format( "( %i ): Es wird versucht ein TimeboyIV anzusprechen.") % error );
                break;

            default:
                text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
                break;
            }
            break;

        default: 
            text = boost::str( boost::format( "( %i ): Nicht eingetragener Fehlerblock für Funktion!") % error );
            break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleHardware::getErrorFunction( module, func, error, text );
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceIDRemote Anzusprechende Geraetenummer.
*/
// Modulbefehl senden und Antwort zurueckgeben
//  deviceIDRemote -> Adresse (Geraetenummer) des Empfaengers
//  module   -> Nummer des anzusprechenden Modules
//  func -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> module-/Funktionsfehler
int Channel4ModuleSystem::call( short context, short module, short func, unsigned char *buf, int &count, int contextValue )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %i, %p, %i, %i )\n", __FILE__, __LINE__, __func__, context, module, func, buf, count, contextValue );

    // Aufruf schaltet den Transparent-Modus ab.
    if ( isTransparencyMode() )
        _transparencyMode = Off;

    int res = Channel4ProtocolCrypt::call( context, module, func, buf, count, contextValue );

    // Bei Verwendung eines Passworts:
    // Wenn das Passwort noch nicht übertragen wurde, wird der Fehler hier abgefangen.
    // Das Passwort wird übertragen und der geforderte Befehl erneut ausgeführt.
    if ( (res & 0xFF) == 28 )
    {
        trace( "%s(%i):INFO(1): %s() [ Passwort wird gefordert. ]\n", __FILE__, __LINE__, __func__, context, module, func, buf, count, contextValue );

        int lRes;
        // Buffer für Folgebefehl zwischenspeichern,
        // da das hinter buf liegende Bytearray durch Aufruf von passwordCheck verändert wird.
        unsigned char data[IV_PACKET_LENGTH_DATA];
        memset( data, 0, sizeof(data) );
        memcpy( data, buf, count );

        // Passwortprüfung durchführen
        if ( (lRes = passwordCheck( getDeviceIdRemote(), (char*)getPasswordCommunication() )) == IV_ERROR_SUCCESS )
        {
            // Passwortprüfung erfolgreich, Parameterdaten wiederherstellen.
            memcpy( buf, data, count );
            // Befehl erneut ausführen.
            return Channel4ProtocolCrypt::call( context, module, func, buf, count, contextValue );
        }
        else
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK );
            // Passwortprüfung fehlgeschlagen, Parameterdaten wiederherstellen.
            memcpy( buf, data, count );
        }

        // Vorherigen Fehler zurückliefern.
        return res;
    }

    return res;
}

//#########################################################
//################# HARDWARE BUGFIX #######################
//#########################################################

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Channel4ModuleSystem::getTypeDisplay( short deviceId, unsigned char &val, int checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    char version[64];
    memset( version, 0, sizeof(version) );
    // Befindet sich im Bootloader?
    if ( !checked && (res = getVersion( deviceId, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = getVersionBootloader( deviceId, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( strstr( version, "00.04.00.00" ) || strstr( version, "00.04.00.01" ) )
            {			
                // Auf entsprechende Funktion verzweigen
                return Channel4ModuleHardware::getTypeTransponder( deviceId, val, TRUE );
            }
        }
    }

    return Channel4ModuleHardware::getTypeDisplay( deviceId, val, TRUE );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Channel4ModuleSystem::getTypeKeypad( short deviceId, unsigned char &val, int checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    char version[64];
    memset( version, 0, sizeof(version) );
    // Befindet sich im Bootloader?
    if ( !checked && (res = getVersion( deviceId, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = getVersionBootloader( deviceId, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( strstr( version, "00.04.00.00" ) || strstr( version, "00.04.00.01" ) )
            {			
                // Auf entsprechende Funktion verzweigen
                return Channel4ModuleHardware::getTypeDisplay( deviceId, val, TRUE );
            }
        }
    }

    return Channel4ModuleHardware::getTypeKeypad( deviceId, val, TRUE );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und stellt einen Bugfix da.
*
*/
int Channel4ModuleSystem::getTypeTransponder( short deviceId, unsigned char &val, int checked )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    int res;
    char version[64];
    memset( version, 0, sizeof(version) );
    // Befindet sich im Bootloader?
    if ( !checked && (res = getVersion( deviceId, version )) != IV_ERROR_SUCCESS )
    {
        if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
        {
            // Befehl wird nicht unterstuetzt
            // Geraet befindet sich im Bootloadermodus
            if ( (res = getVersionBootloader( deviceId, version )) != IV_ERROR_SUCCESS )
            {
                return res;
            }

            // Ist eine von den gesuchten Versionen?
            if ( strstr( version, "00.04.00.00" ) || strstr( version, "00.04.00.01" ) )
            {			
                // Auf entsprechende Funktion verzweigen
                return Channel4ModuleHardware::getTypeKeypad( deviceId, val, TRUE );
            }
        }
    }

    return Channel4ModuleHardware::getTypeTransponder( deviceId, val, TRUE );
}

// ##################################################################
/**
* @brief Überschreibt eine gleichnamige Funktion aus dem zugrundeliegenden Module
*        und ist aufgrund eines Bugfixes hier implementiert.
*
*/
int Channel4ModuleSystem::getHardwareInfo( short deviceId, StructHardwareInfo &hwInfo )
{
    int res = IV_ERROR_SUCCESS;
    bool readOneByOne = true;

    // Struktur initialisieren
    memset( &hwInfo, 0xFF, sizeof(StructHardwareInfo) );
    // Administrationsflags ruecksetzen
    hwInfo.administrationFlags = 0;

    if ( (res = Channel4ModuleHardware::getHardwareInfo( deviceId, hwInfo )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        // Oder die erwartetete Datenstruktur passt nicht.
        if ( !(((res >> 8) & 0xFF) == 2) && !(((res >> 8) & 0xFF) == 3) && !(((res >> 8) & 0xFF) == 4) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Daten bereits erfolgreich ermittelt, nun nicht mehr eines nach dem anderen lesen.
        readOneByOne = false;
        // Bei Setupoptionen nur die der Klasse bekannten weiterreichen.
        hwInfo.optionsSetup &= IV_VALUE_SETUP_OPTION_MASK;
        // Bei den Bezahloptionen nur die der Klasse bekannten weiterreichen.
        if ( hwInfo.optionsPayable == 0xFF )
        {
            // Wert nicht initialisiert.
            hwInfo.optionsPayable = 0;
        }
        else
        {
            hwInfo.optionsPayable &= IV_VALUE_PAYABLE_OPTION_MASK;
        }
    }

    //#####################################################
    // Version der Hardwareinfo ermitteln
    if ( readOneByOne && (res = getHWConfig( deviceId, hwInfo.version )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Liegt keine Hardwareinfo vor?
    if ( hwInfo.version == 0xFF )
        hwInfo.version = 0;

    // Geraetetyp ermitteln
    if ( readOneByOne && (res = getTypeDevice( deviceId, hwInfo.typeDevice )) != IV_ERROR_SUCCESS )
    {		
        // Wenn der Modulfehler -Funktion unbekannt- ist,
        // dann liegt eine FW 04.01.01.<k vor.
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hwInfo.typeDevice == 0xFF )
        hwInfo.typeDevice = IV_VALUE_TYPE_DEVICE_PZE;

    // Boardtyp ermitteln
    if ( readOneByOne && (res = getTypeBoard( deviceId, hwInfo.typeBoard )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeBoard == 0xFF )
        hwInfo.typeBoard = IV_VALUE_TYPE_BOARD_1_4;

    // RS485 ermitteln
    if ( readOneByOne && (res = getComponentRS485_0( deviceId, hwInfo.componentRS485_0 )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hwInfo.componentRS485_0 == 0xFF )
        hwInfo.componentRS485_0 = 0;

    // TCPIP ermitteln
    if ( readOneByOne && (res = getTypeTCPIP( deviceId, hwInfo.typeTCPIP )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hwInfo.typeTCPIP == 0xFF )
        hwInfo.typeTCPIP = IV_VALUE_TYPE_TCPIP_NONE;

    // ModemMC35i ermitteln
    if ( readOneByOne && (res = getTypeModemMC( deviceId, hwInfo.typeModemMC )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hwInfo.typeModemMC == 0xFF )
        hwInfo.typeModemMC = IV_VALUE_TYPE_MODEM_MC_NONE;

    // RS485 (intern) ermitteln
    if ( readOneByOne && (res = getComponentRS485_1( deviceId, hwInfo.componentRS485_1 )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzten.
    if ( hwInfo.componentRS485_1 == 0xFF )
        hwInfo.componentRS485_1 = 0;

    // Typ ermitteln (erst ab 04.01.01.p / Bootloader 4.0)
    if ( readOneByOne && (res = getTypeDisplay( deviceId, hwInfo.typeDisplay )) != IV_ERROR_SUCCESS )
    {
        // Wenn der Modulfehler -Funktion unbekannt- ist,
        // dann liegt eine FW 04.01.01.<p vor.
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeDisplay == 0xFF )
        hwInfo.typeDisplay = IV_VALUE_TYPE_DISPLAY_SMALL;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeKeypad(deviceId, hwInfo.typeKeypad)) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeKeypad == 0xFF )
        hwInfo.typeKeypad = IV_VALUE_TYPE_KEYPAD_9;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeTransponder( deviceId, hwInfo.typeTransponder )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeTransponder == 0xFF )
        hwInfo.typeTransponder = IV_VALUE_TYPE_TRANSPONDER_TSR32;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeMMC( deviceId, hwInfo.typeMMC )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeMMC == 0xFF )
        hwInfo.typeMMC = IV_VALUE_TYPE_MMC_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeRadio( deviceId, hwInfo.typeRadio )) != IV_ERROR_SUCCESS )
    {	
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeRadio == 0xFF )
        hwInfo.typeRadio = IV_VALUE_TYPE_RADIO_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeGPS( deviceId, hwInfo.typeGPS )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeGPS == 0xFF )
        hwInfo.typeGPS = IV_VALUE_TYPE_GPS_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = getComponentUSB( deviceId, hwInfo.componentUSB )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.componentUSB == 0xFF )
        hwInfo.componentUSB = 0;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeDCF77( deviceId, hwInfo.typeDCF77 )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeDCF77 == 0xFF )
        hwInfo.typeDCF77 = IV_VALUE_TYPE_DCF77_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = getComponentInAnalog( deviceId, hwInfo.componentInAnalog )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.componentInAnalog == 0xFFFFFFFF )
        hwInfo.componentInAnalog = IV_VALUE_COMPONENT_IN_ANALOG_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = getComponentInDigital( deviceId, hwInfo.componentInDigital )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.componentInDigital == 0xFFFFFFFF )
        hwInfo.componentInDigital = IV_VALUE_COMPONENT_IN_DIGITAL_NONE;

    // Komponende ermitteln
    if ( readOneByOne && (res = getComponentOutDigital( deviceId, hwInfo.componentOutDigital )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.componentOutDigital == 0xFFFFFFFF )
        hwInfo.componentOutDigital = IV_VALUE_COMPONENT_OUT_DIGITAL_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeFingerprint( deviceId, hwInfo.typeFingerprint )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn nicht initialisiert, dann Default setzen.
    if ( hwInfo.typeFingerprint == 0xFF )
        hwInfo.typeFingerprint = IV_VALUE_TYPE_FINGERPRINT_NONE;

    if ( readOneByOne )
    {
        hwInfo.typeExternalReader = 0xFF;
        // Typ ermitteln
        if ( (res = getTypeExternalReader( deviceId, hwInfo.typeExternalReader )) != IV_ERROR_SUCCESS )
        {		
            // Nicht der Modulfehler -Funktion unbekannt-
            if ( !(((res >> 8) & 0xFF) == 2) )
                return res;

            // Fehlerobjekt zurueck setzen
            res = IV_ERROR_SUCCESS;
        }
    }

    // Wenn ZK Option vorhanden
    if ( hwInfo.componentRS485_1 )
    {
        // aber der Externe Lesertyp undefiniert ist,
        // dann wird von TS-Lesern ausgegangen.
        if ( hwInfo.typeExternalReader == 0xFF )
            hwInfo.typeExternalReader = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
    }
    else
    {
        // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
        // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
        // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
        // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
        // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
        // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
        // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
        // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
        // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
        // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
        // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
        // IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER und Board 1.0
        bool evoLine = hwInfo.typeDevice == 0x0a 
            || hwInfo.typeDevice == 0x0b 
            || hwInfo.typeDevice == 0x0c 
            || hwInfo.typeDevice == 0x0d 
            || hwInfo.typeDevice == 0x0e
            || hwInfo.typeDevice == 0x0F
            || hwInfo.typeDevice == 0x10
            || hwInfo.typeDevice == 0x11
            || hwInfo.typeDevice == 0x12
            || hwInfo.typeDevice == 0x13
            || hwInfo.typeDevice == 0x14
            || hwInfo.typeDevice == 0x15
            || hwInfo.typeDevice == 0x16;
        if ( !evoLine && hwInfo.typeBoard < 0x40 )
        {
            // Die Abhängigkeit dieser Option von hwInfo.componentRS485_1 ist nur bei den älteren Geräten vorhanden.
            // Keine Externen leser
            hwInfo.typeExternalReader = IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED;
        }
    }
    // BUGFIX
    // Eine Firmware die die Serien TS und Protect unterstützt setzt als unterstützte Serien
    // TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
    // so dass der Wert der Hardwareinfo eindeutig ist.
    // Werden beide Flags vorgefunden wird von TS ausgegangen,
    // wird Protect alleine vorgefunden wird von PHG alleine ausgegangen.
    if ( hwInfo.typeExternalReader & 0x01 && hwInfo.typeExternalReader & 0x04 )
        hwInfo.typeExternalReader = 0x01;
    else if ( hwInfo.typeExternalReader & 0x04 )
        // BUGFIX
        // Eine Firmware < 04.01.04.43 stellt fest ihre unterstuetzte Serie und verwendet 4 fuer PHG.
        hwInfo.typeExternalReader = 0x02;

    // Option ermitteln
    if ( readOneByOne && (res = getOptionIOModule( deviceId, hwInfo.optionIOModule )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.optionIOModule == 0xFF )
        hwInfo.optionIOModule = IV_VALUE_OPTION_IOMODULE_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = getOptionTBAdaption( deviceId, hwInfo.optionTBAdaption )) != IV_ERROR_SUCCESS )
    {		
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist und ein Mobil-MasterIV vorliegt, dann aktiv ansonsten deaktiviert.
    if ( hwInfo.optionTBAdaption == 0xFF )
    {
        if ( hwInfo.typeDevice == IV_VALUE_TYPE_DEVICE_MOBIL )
            hwInfo.optionTBAdaption = IV_VALUE_OPTION_TBADAPTION;
        else
            hwInfo.optionTBAdaption = IV_VALUE_OPTION_TBADAPTION_NONE;
    }

    // Option ermitteln
    if ( readOneByOne && (res = getOptionComServer( deviceId, hwInfo.optionComserver )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {	
        // BUGFIX:
        // Dieser Befehl existiert bereits in der Firmware obwohl die Option noch nicht unterstützt wird.
        // Er gehört zur Gruppe 2 der Zusatzoptionen liegt aber per Befehl in der Gruppe 1
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.optionComserver == 0xFF )
        hwInfo.optionComserver = IV_VALUE_OPTION_COMSERVER_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = getOptionTPWrite( deviceId, hwInfo.optionTPWrite )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 2 verfuegbar
        hwInfo.administrationFlags |= HIAF_HAS_OPTIONS2;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.optionTPWrite == 0xFF )
        hwInfo.optionTPWrite = IV_VALUE_OPTION_TPWRITE_NONE;

    // Option ermitteln
    if ( readOneByOne && (res = getOptionHTTP( deviceId, hwInfo.optionHttp )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von aktiviert ausgegangen.
    if ( hwInfo.optionHttp == 0xFF )
    {
        hwInfo.optionHttp = IV_VALUE_OPTION_HTTP_NONE;

        if ( hwInfo.typeModemMC )
            hwInfo.optionHttp = IV_VALUE_OPTION_HTTP;
    }

    // Option ermitteln
    if ( readOneByOne && (res = getOptionLanguage( deviceId, hwInfo.optionLanguage )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von aktiviert ausgegangen.
    if ( hwInfo.optionLanguage == 0xFF )
        hwInfo.optionLanguage = IV_VALUE_OPTION_LANGUAGE;


    // Option ermitteln
    if ( readOneByOne && (res = getOptionBiokeyUpdate( deviceId, hwInfo.optionBiokeyUpdate )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn die Option undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.optionBiokeyUpdate == 0xFF )
        hwInfo.optionBiokeyUpdate = IV_VALUE_OPTION_BIOKEY_UPDATE_NONE;

    unsigned short mask = 0xFFFF;
    // Option ermitteln
    if ( readOneByOne && (res = getSetupOptions( deviceId, mask, hwInfo.optionsSetup )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 3 verfuegbar
        hwInfo.administrationFlags |= HIAF_HAS_OPTIONS3;
    }

    // Wenn die Option undefiniert ist dann wird von maximal moeglich ausgegangen.
    //	if (hwInfo.optionsSetup == 0xFFFF) {
    //#ifdef _DEBUG
    hwInfo.optionsSetup = (mask & hwInfo.optionsSetup);
    //#endif
    //	}

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeTouch( deviceId, hwInfo.typeTouch )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        hwInfo.administrationFlags |= HIAF_HAS_OPTIONS4;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.typeTouch == 0xFF )
        hwInfo.typeTouch = IV_VALUE_TYPE_TOUCH_NONE;

    // Typ ermitteln
    if ( readOneByOne && (res = getTypeSensor( deviceId, hwInfo.typeSensor )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.typeSensor == 0xFF )
        hwInfo.typeSensor = IV_VALUE_TYPE_SENSOR_NONE;

    // Komponente ermitteln
    if ( readOneByOne && (res = getComponentBattery( deviceId, hwInfo.componentBattery )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.componentBattery == 0xFF )
        hwInfo.componentBattery = 0;

    // Komponente ermitteln
    if ( readOneByOne && (res = getComponentCharge( deviceId, hwInfo.componentCharge )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }

    // Wenn der Typ undefiniert ist dann wird von deaktiviert ausgegangen.
    if ( hwInfo.componentCharge == 0xFF )
        hwInfo.componentCharge = 0;

    // Options ermitteln
    unsigned char po_mask = IV_VALUE_PAYABLE_OPTION_ALL;
    if ( readOneByOne && (res = getPayableOptions( deviceId, po_mask, hwInfo.optionsPayable )) != IV_ERROR_SUCCESS )
    {
        // Nicht der Modulfehler -Funktion unbekannt-
        if ( !(((res >> 8) & 0xFF) == 2) )
            return res;

        // Fehlerobjekt zurueck setzen
        res = IV_ERROR_SUCCESS;
    }
    else
    {
        // Fasthack um festzustellen, dass die Optionen der Gruppe 5 verfuegbar
        hwInfo.administrationFlags |= HIAF_HAS_OPTIONS5;
    }

    return res;
}

// ##################################################################
/**
* @brief Löst die Hardwareinfo in seine String-Representation auf.
*
*/
int Channel4ModuleSystem::getInfos( StructHardwareInfo hwInfo, std::string &info )
{
    std::string tmp;
    unsigned int bit;
    int res = 1;

	tmp.clear();

    info.append( "Hardwareinformation\r\n" );
    info.append( "------------------------------------------\r\n" );
    info.append( "\r\nBasis\r\n----------\r\n" );
    info.append( "Geraet: " );
    // Moegliche Geraetetypen
    switch ( hwInfo.typeDevice ) 
    {
    case IV_VALUE_TYPE_DEVICE_PZE:                         tmp.append( IV_VALUE_TYPE_DEVICE_PZE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK:                          tmp.append( IV_VALUE_TYPE_DEVICE_ZK_NAME );  break;
    case IV_VALUE_TYPE_DEVICE_BDE:                         tmp.append( IV_VALUE_TYPE_DEVICE_BDE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL:                       tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MDE:                         tmp.append( IV_VALUE_TYPE_DEVICE_MDE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_AE:                          tmp.append( IV_VALUE_TYPE_DEVICE_AE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY:                     tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_EXKLUSIV:                    tmp.append( IV_VALUE_TYPE_DEVICE_EXKLUSIV_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_FLEX:                        tmp.append( IV_VALUE_TYPE_DEVICE_FLEX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE:           tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TERMINAL_2_8:                tmp.append( IV_VALUE_TYPE_DEVICE_TERMINAL_2_8_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TERMINAL_4_3:                tmp.append( IV_VALUE_TYPE_DEVICE_TERMINAL_4_3_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7:     tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10:      tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7:            tmp.append( IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_IO_BOX:                      tmp.append( IV_VALUE_TYPE_DEVICE_IO_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_BOX:                      tmp.append( IV_VALUE_TYPE_DEVICE_ZK_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL_BOX:                   tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_BOX_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK:              tmp.append( IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK:                tmp.append( IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_KNOTEN:                   tmp.append( IV_VALUE_TYPE_DEVICE_ZK_KNOTEN_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_ZK_TERMINAL:                 tmp.append( IV_VALUE_TYPE_DEVICE_ZK_TERMINAL_NAME ); break;
    case IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER:             tmp.append( IV_VALUE_TYPE_DEVICE_SAMS_OCL_READER_NAME ); break;
    default:                                               tmp.append( "Undefiniert" ); break;
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    info.append( "Platinenversion: " );
    // Boardtyp pruefen
    switch ( hwInfo.typeBoard )
    {
    case IV_VALUE_TYPE_BOARD_1_0: tmp.append( "1.0" ); break;
    case IV_VALUE_TYPE_BOARD_1_1: tmp.append( "1.1" ); break;
    case IV_VALUE_TYPE_BOARD_1_2: tmp.append( "1.2" ); break;
    case IV_VALUE_TYPE_BOARD_1_3: tmp.append( "1.3" ); break;
    case IV_VALUE_TYPE_BOARD_1_4: tmp.append( "1.4" ); break;
    case IV_VALUE_TYPE_BOARD_1_5: tmp.append( "1.5" ); break;
    case IV_VALUE_TYPE_BOARD_2_0: tmp.append( "2.0" ); break;
    case IV_VALUE_TYPE_BOARD_2_1: tmp.append( "2.1" ); break;
    case IV_VALUE_TYPE_BOARD_2_3: tmp.append( "2.3" ); break;
    case IV_VALUE_TYPE_BOARD_2_4: tmp.append( "2.4" ); break;
    case IV_VALUE_TYPE_BOARD_3_0: tmp.append( "3.0" ); break;
    case IV_VALUE_TYPE_BOARD_3_1: tmp.append( "3.1" ); break;
	case IV_VALUE_TYPE_BOARD_3_4: tmp.append( "3.4" ); break;
	case IV_VALUE_TYPE_BOARD_4_0: tmp.append( "4.0" ); break;
	case IV_VALUE_TYPE_BOARD_4_1: tmp.append( "4.1" ); break;
	case IV_VALUE_TYPE_BOARD_4_2: tmp.append( "4.2" ); break;
    case IV_VALUE_TYPE_BOARD_4_3: tmp.append( "4.3" ); break;
    case IV_VALUE_TYPE_BOARD_4_4: tmp.append( "4.4" ); break;
    default:                      tmp.append( "Undefiniert" ); break;
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    /////////////////////////////////////////////
    // Hauptkommunikationen
    info.append( "\r\nHauptkommunikationen\r\n------------------------------\r\n" );

    // Funk pruefen
    info.append( "Funk: " );
    if ( hwInfo.typeRadio )
    {
        switch ( hwInfo.typeRadio )
        {
        case IV_VALUE_TYPE_RADIO_WRAPTHOR: tmp.append( "Bluetooth" ); break;
        case IV_VALUE_TYPE_RADIO_ZIGBEE:   tmp.append( "Zigbee" ); break;
        default:                           tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
    tmp.clear();

    // ModemMCxx pruefen
    info.append( "Mobilfunk: " );
    if ( hwInfo.typeModemMC )
    {
        switch ( hwInfo.typeModemMC )
        {
        case IV_VALUE_TYPE_MODEM_MC_39: tmp.append( "Siemens MC39" ); break;
        case IV_VALUE_TYPE_MODEM_MC_55: tmp.append( "Siemens MC55" ); break;
        case IV_VALUE_TYPE_MODEM_RS9110_N_11_24: tmp.append( "Redpine RS9110-N-11-24" ); break;
        default: tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // RS485 pruefen
    tmp = str( format( "RS485: %1%\r\n" ) % (hwInfo.componentRS485_0 ? "Ja" : "Nein" ) );
    info.append( tmp );
	tmp.clear();

    // TCPIP pruefen
    info.append( "TCPIP: " );
    if ( hwInfo.typeTCPIP )
    {
        switch ( hwInfo.typeTCPIP )
        {
        case IV_VALUE_TYPE_TCPIP_XPORT:     tmp.append( "Ja"); break;
        case IV_VALUE_TYPE_TCPIP_MATCHPORT: tmp.append( "Ja, über WLAN"); break;
        default:                            tmp.append( "Undefiniert"); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // USB pruefen
    tmp = str( format( "USB: %1%\r\n" ) % (hwInfo.componentUSB ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    /////////////////////////////////////////////
    // Zusatzoptionen
    info.append( "\r\nWaehlbare Optionen der Firmware\r\n------------------------------\r\n");

    // Zusatzoption eingestellt?
	tmp = str( format( "Anbindung IO-Modul: %1%\r\n" ) % (hwInfo.optionIOModule ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "Anbindung TimeboyIII: %1%\r\n" ) % (hwInfo.optionTBAdaption ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "Com-Servermodus: %1%\r\n" ) % (hwInfo.optionComserver ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "HTTP-Protokoll: %1%\r\n" ) % (hwInfo.optionHttp ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "Transponder ueber Transpondermenue schreiben: %1%\r\n" ) % (hwInfo.optionTPWrite ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "Textdaten benötigt: %1%\r\n" ) % (hwInfo.optionLanguage ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzoption eingestellt?
	tmp = str( format( "Biokeymodulupdate möglich: %1%\r\n" ) % (hwInfo.optionBiokeyUpdate ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Setupoptionen (intern) ermitteln
	tmp = str( format( "Setupoptionen (0x%04X): " ) % hwInfo.optionsSetup );
    info.append( tmp );
	tmp.clear();
    for ( bit = 1; ; bit <<= 1 )
    {
        // Nach maximal bekannter Option abbrechen
        if ( !(bit & IV_VALUE_SETUP_OPTION_MASK) )
			break;

        if ( tmp.size() ) 
            tmp.append( ", ");

        switch ( bit )
        {
        case IV_VALUE_SETUP_OPTION_IOCONTROL:
            tmp.append( "Signalverarbeitung( " ); 
			if ( bit & hwInfo.optionsSetup )
				tmp.append( "Ja )" );
			else
				tmp.append( "Nein )" );

            break;

        default:
            tmp.append( "Undefiniert" );
            break;
        }
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Bezahlbare Optionen ermitteln
    tmp = str( format( "Erweiterte Optionen (0x%02X): ") % hwInfo.optionsPayable );
    info.append( tmp );
    tmp.clear();
    for ( bit = 1; ; bit <<= 1 )
    {
        // Nach maximal bekannter Option abbrechen
        if ( !(bit & IV_VALUE_PAYABLE_OPTION_MASK) )
			break;

        if ( tmp.size() ) 
            tmp.append( ", " );

        switch ( bit )
        {
        case IV_VALUE_PAYABLE_OPTION_DATA_ON_CARD:
            tmp.append( "Data on Card( " ); 
			if ( bit & hwInfo.optionsPayable )
				tmp.append( "Ja )");
			else
				tmp.append( "Nein )");

            break;
        default:
            tmp.append( "Undefiniert" );
            break;
        }
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    /////////////////////////////////////////////
    // Zusatzbaugruppen
    info.append( "\r\nZusatzbaugruppen\r\n------------------------------\r\n");
    tmp.clear();

    // Display pruefen
    info.append( "Anzeige: " );
    if ( hwInfo.typeDisplay )
    {
        switch ( hwInfo.typeDisplay )
        {
        case IV_VALUE_TYPE_DISPLAY_SMALL:         tmp.append( "4 Zoll Monochrome" ); break;
        case IV_VALUE_TYPE_DISPLAY_BIG:           tmp.append( "6 Zoll Monochrome" ); break;
        case IV_VALUE_TYPE_DISPLAY_SMALLER:       tmp.append( "1,9 Zoll Color" ); break;
        case IV_VALUE_TYPE_DISPLAY_TIMEBOY_2ZOLL: tmp.append( "2 Zoll Color" ); break;
        default:                                  tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Touch pruefen
    info.append( "Touch: " );
    if ( hwInfo.typeTouch )
    {
        switch ( hwInfo.typeTouch )
        {
        case IV_VALUE_TYPE_TOUCH_SMALL:   tmp.append( "4 Zoll" ); break;
        case IV_VALUE_TYPE_TOUCH_BIG:     tmp.append( "6 Zoll" ); break;
        default:                          tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Sensor pruefen
    info.append( "Sensor: " );
    if ( hwInfo.typeSensor )
    {
        switch ( hwInfo.typeSensor )
        {
        case IV_VALUE_TYPE_SENSOR_VIBRATION:   tmp.append( "Vibration" ); break;
        case IV_VALUE_TYPE_SENSOR_SPEEDUP:     tmp.append( "Beschleunigung" ); break;
        default:                               tmp.append( "Undefiniert" ); break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();


    // DigitalOut pruefen
    info.append( "Ausgaenge Digital: " );
    if ( hwInfo.componentOutDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hwInfo.componentOutDigital )
            {
                if ( tmp.size() )
                    tmp.append( ", ");

                switch ( bit )
                { // Digitale Ausgaenge
                case 0x01: tmp.append( "dO1" ); break;
                case 0x02: tmp.append( "dO2" ); break;
                case 0x04: tmp.append( "dO3" ); break;
                case 0x08: tmp.append( "dO4" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
				break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // AnalogIn pruefen
    info.append( "Eingaenge Analog: " );
    if ( hwInfo.componentInAnalog )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hwInfo.componentInAnalog )
            {
                if ( tmp.size() )
                    tmp.append( ", ");

                switch ( bit )
                { // Analoge Eingaenge
                case 0x01: tmp.append( "aI1" ); break;
                case 0x02: tmp.append( "aI2" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
				break;
        }
    }
    else 
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // DigitalIn pruefen
    info.append( "Eingaenge Digital: " );
    if ( hwInfo.componentInDigital )
    {
        for ( bit = 1; ; bit <<= 1 )
        {
            if ( bit & hwInfo.componentInDigital )
            {
                if ( tmp.size() )
                    tmp.append( ", " );

                switch ( bit )
                { // Digitale Eingaenge
                case 0x01: tmp.append( "dI1" ); break;
                case 0x02: tmp.append( "dI2" ); break;
                case 0x04: tmp.append( "dI3" ); break;
                case 0x08: tmp.append( "dI4" ); break;
                case 0x10: tmp.append( "dI5" ); break;
                case 0x20: tmp.append( "dI6" ); break;
                case 0x40: tmp.append( "dI7" ); break;
                case 0x80: tmp.append( "dI8" ); break;
                default:   tmp.append( "Undefiniert" ); break;
                }
            }

            if ( bit == 0x8000000 )
				break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Zusatzkomponente bestückt?
    tmp = str( format( "Batterie bestückt: %1%\r\n") % (hwInfo.componentBattery ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Zusatzkomponente bestückt?
    tmp = str( format( "Ladeschaltung bestückt: %1%\r\n") % (hwInfo.componentCharge ? "Ja" : "Nein") );
    info.append( tmp );
	tmp.clear();

    // Fingerprint pruefen
    info.append( "Fingerprint: " );
    if ( hwInfo.typeFingerprint )
    {
        switch ( hwInfo.typeFingerprint )
        {
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_3000: tmp.append( "Ja (3000)" ); break;
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4000: tmp.append( "Ja (4000)" ); break;
        case IV_VALUE_TYPE_FINGERPRINT_BIOKEY_4020: tmp.append( "Ja (4020)" ); break;
        default:                                    tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // DCF77 pruefen
    info.append( "Funkuhr/Barcode (intern): " );
    if ( hwInfo.typeDCF77 )
    {
        switch ( hwInfo.typeDCF77 )
        {
        case IV_VALUE_TYPE_DCF77_HKW:            tmp.append( "Funkuhr" ); break;
        case IV_VALUE_TYPE_DCF77_BARCODE:        tmp.append( "Barcode Typ 1" ); break;
        case IV_VALUE_TYPE_DCF77_BARCODE_ZEBEX:  tmp.append( "Barcode Typ 2" ); break;
        default:                                 tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // GPS pruefen
    info.append( "GPS: " );
    if ( hwInfo.typeGPS )
    {
        switch ( hwInfo.typeGPS )
        {
        case IV_VALUE_TYPE_GPS_GN80:       tmp.append( "Ja, Typ 1" ); break;
        case IV_VALUE_TYPE_GPS_5H:         tmp.append( "Ja, Typ 2" ); break;
        case IV_VALUE_TYPE_GPS_MOUSE_5H:   tmp.append( "Ja, Typ 3" ); break;
        default:                           tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // MMC pruefen
    info.append( "MM Card: " );
    if ( hwInfo.typeMMC )
    {
        switch( hwInfo.typeMMC )
        {
        case IV_VALUE_TYPE_MMC_EXTENSION: tmp.append( "Speichererweiterung" ); break;
        case IV_VALUE_TYPE_MMC_DISK:      tmp.append( "Wechseldatenträger" ); break;
        case IV_VALUE_TYPE_MMC_USB:       tmp.append( "USB-Host Wechseldatenträger" ); break;
        default:                          tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Tastatur pruefen
    info.append( "Tastatur: " );
    if ( hwInfo.typeKeypad )
    {
        switch ( hwInfo.typeKeypad )
        {
        case IV_VALUE_TYPE_KEYPAD_9:  tmp.append( "9 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_58: tmp.append( "58 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_29: tmp.append( "29 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_7:  tmp.append( "7 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_25: tmp.append( "25 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_20: tmp.append( "20 Tasten" ); break;
        case IV_VALUE_TYPE_KEYPAD_3:  tmp.append( "3 Tasten" ); break;
        default:                      tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // Transponder pruefen
    info.append( "Transponderleser: " );
    if ( hwInfo.typeTransponder )
    {
        switch ( hwInfo.typeTransponder )
        {
        case IV_VALUE_TYPE_TRANSPONDER_TSR30:          tmp.append( "[EM4102, Hitag2]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TSR32:          tmp.append( "[EM4102, EM4450, Hitag]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_PRIMO100:       tmp.append( "[Legic]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_MIFARE:         tmp.append( "[Mifare]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_EXTERN: tmp.append( "[IButton (Extern)]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS:    tmp.append( "[SimonsVoss SmartRelais]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_NEDAP:          tmp.append( "[Nedap]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_IBUTTON_INTERN: tmp.append( "[IButton (Intern)]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_HID_PROXKEY:    tmp.append( "[HID ProxKey]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MULTI_ISO: tmp.append( "[Mifare Desfire]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_PRIMO130:       tmp.append( "[Legic Advant]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TSR21:          tmp.append( "[EM4102, EM4450, Hitag]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_HID_ICLASS:     tmp.append( "[HID iClass]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TB_MIFARE:      tmp.append( "[Timeboy Mifare]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TB_LEGIC:       tmp.append( "[Timeboy Legic]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_SMARTRELAIS_G2: tmp.append( "[SimonsVoss SmartRelais G2]" ); break;
        case IV_VALUE_TYPE_TRANSPONDER_TWN3_MIFARE_NFC:tmp.append( "[Mifare NFC (ISO14443)]" ); break;
        default:                                       tmp.append( "[Undefiniert]" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    // RS485 (intern) ermitteln
    info.append( "Zutrittskontrolle: " );
    if ( hwInfo.componentRS485_1 )
    {
        switch ( hwInfo.typeExternalReader )
        {
        case IV_VALUE_TYPE_EXTERNAL_READER_GIS:     tmp.append( "TS-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_PHG:     tmp.append( "PHG-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_PROTECT: tmp.append( "Protect-Serie" ); break;
        case IV_VALUE_TYPE_EXTERNAL_READER_IDTRONIC: tmp.append( "ID Tronic-Serie" ); break;
        case 0xFF:                                  tmp.append( "Unbekannt" ); break;
        default:                                    tmp.append( "Undefiniert" ); break;
        }
    }
    else
    {
        tmp.append( "Nein" );
    }
    tmp.append( "\r\n" );
    info.append( tmp );
	tmp.clear();

    return res;
}

//#########################################################
//####################### SYSTEM ##########################
//#########################################################

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::remoteCall( short module, short func, unsigned char *data, int &count )
{
    if ( module == IV_MODULE_SYSTEM )
    {
        switch ( func )
        {
            // Ermitteln der Firmwareversion
        case IV_FUNCTION_GET_VERSION:
            return getVersion(getDeviceId(), (char*)data);

            // Setzen/Ermitteln von: SerialNum
        case IV_FUNCTION_SET_SERIAL:
            return setSerial(getDeviceId(), *(unsigned int*)data, *(unsigned int*)&data[4]);

        case IV_FUNCTION_GET_SERIAL:
            return getSerial(getDeviceId(), (unsigned int&)data);

            // Setzen/Ermitteln von: DateTime (YYYY-MM-DD HH:MM:SS\0)
        case IV_FUNCTION_SET_RTC:
            return setRTC(getDeviceId(), (char*)data);

        case IV_FUNCTION_GET_RTC:
            return getRTC(getDeviceId(), (char*)data);

            // Setzen/Ermitteln von: Variable (Systemvariablen GPRS) 
        case IV_FUNCTION_SET_SYSTEM_VARIABLE:
            {
                char *name = (char*)data;
                char *val = (char*)&data[strlen(name) + 1];
                return setSystemVariable(getDeviceId(), name, val);                                      
            }

        case IV_FUNCTION_GET_SYSTEM_VARIABLE:
            {
                char *name = (char*)data;
                char *val = (char*)&data[strlen(name) + 1];
                int res = getSystemVariable(getDeviceId(), name, val);
                if (res == IV_ERROR_SUCCESS) {
                    count += (int)strlen(val) + 1;
                }
                return res;
            }
            // Setzen/Ermitteln von: Relaiszustand 
            //		case IV_FUNCTION_SET_RELAY: return CallSetRelay(data, count);
            //		case IV_FUNCTION_GET_RELAY: return CallGetRelay(data, count);

            // Schreiben/Lesen von: Externen Flash
            //		case IV_FUNCTION_SET_FLASH: return CallSetFlash(data, count);
            //		case IV_FUNCTION_GET_FLASH: return CallGetFlash(data, count);

            // Schreiben/Lesen von: EEPROM
            //		case IV_FUNCTION_SET_EEPROM: return CallSetEEPROM(data, count);
            //		case IV_FUNCTION_GET_EEPROM: return CallGetEEPROM(data, count);

            // Schreiben/Lesen von: Program-Flash schreiben/lesen
            //		case IV_FUNCTION_SET_PROGRAM: return CallSetProgram(data, count);
            //		case IV_FUNCTION_GET_PROGRAM: return CallGetProgram(data, count);

            // Starten/Beenden von: Segmentuebertragung (inklusive aktiver Antwort)
            //		case IV_FUNCTION_BEGIN_SET_SEGMENT: return beginSetSegment();
        case IV_FUNCTION_BEGIN_SET_SEGMENT_OK:
            return beginSetSegmentOK(getDeviceId());

            //		case IV_FUNCTION_END_SET_SEGMENT: return CallEndSetSegment(data, count);
        case IV_FUNCTION_END_SET_SEGMENT_OK:
            return endSetSegmentOK(getDeviceId());

            // Ermitteln von: Bootloaderversion
        case IV_FUNCTION_GET_VERSION_BOOTLOADER:
            return getVersionBootloader(getDeviceId(), (char*)data);

            // Durchfuehren eines Resets der Speicherbereiche
            //		case IV_FUNCTION_SYSTEM_RESET: return CallSystemReset(data, count);

            // Ins Bios wechseln
        case IV_FUNCTION_ENTER_BIOS:
            return enterBios(getDeviceId());

            // Bioswechsel erfolgt
        case IV_FUNCTION_ENTERED_BIOS:
            return enteredBios(getDeviceId());

            // Bios verlassen (aenderungen uebernehmen
        case IV_FUNCTION_EXIT_BIOS:
            return exitBios(getDeviceId());

            // Biosverlassen erfolgt
        case IV_FUNCTION_EXITED_BIOS:
            return exitedBios(getDeviceId());

            // Setzt/liest das zu verwendende Interface
            //		case IV_FUNCTION_SET_INTERFACE: return CallSetInterface(data, count);
            //		case IV_FUNCTION_GET_INTERFACE: return CallGetInterface(data, count);

            // Setzt/liest die zu verwendende BaudRate
            //		case IV_FUNCTION_SET_BAUD: return CallSetBaud(data, count);
            //		case IV_FUNCTION_GET_BAUD: return CallGetBaud(data, count);

            // Setzt/liest die zu verwendende DeviceID
            //		case IV_FUNCTION_SET_DEVICE_ID: return CallSetDeviceIDBios(data, count);
            //		case IV_FUNCTION_GET_DEVICE_ID: return CallGetDeviceIDBios(data, count);

            // Befehl der beim Geraetestart den Bootloader des ZK - Master IV wiederherstellt.
            //		case IV_FUNCTION_ENTER_BOOTLOADER: return CallEnterBootloader(data, count);

            // In Monitor Mode von XPort wechselt
            //		case IV_FUNCTION_ENTER_XPORT_MONITOR: return CallEnterXPortMonitor(data, count);
            // Antwort in Monitor Mode gewechselt
        case IV_FUNCTION_ENTER_XPORT_MONITOR_OK: return enterXPortMonitorOK(getDeviceId());

            // Monitorbefehl an XPort und Antwort erhalten.
            //		case IV_FUNCTION_CMD_XPORT: return CallCommandXPort(data, count);

            // In Monitor Mode von XPort wechselt
            //		case IV_FUNCTION_EXIT_XPORT_MONITOR: return CallExitXPortMonitor(data, count);
            // Antwort in Monitor Mode gewechselt
        case IV_FUNCTION_EXIT_XPORT_MONITOR_OK: return exitXPortMonitorOK(getDeviceId());


            //		case IV_FUNCTION_BEGIN_GET_SEGMENT: return CallBeginGetSegment(data, count);
        case IV_FUNCTION_BEGIN_GET_SEGMENT_OK: return beginGetSegmentOK(getDeviceId());

            //		case IV_FUNCTION_END_GET_SEGMENT: return CallEndGetSegment(data, count);

        case IV_FUNCTION_END_GET_SEGMENT_OK: return endGetSegmentOK(getDeviceId());
        case IV_FUNCTION_SYSTEM_RESET_OK: return systemResetOK(getDeviceId());
            //		case IV_FUNCTION_OPEN_COM_SERVER_MODE:  return openComServerMode(GetDeviceID());
            //		case IV_FUNCTION_CLOSE_COM_SERVER_MODE: return closeComServerMode(GetDeviceID()=);
            //		case IV_FUNCTION_GET_INFO:
        case IV_FUNCTION_OPEN_COM_SERVER_MODE_OK: return openComServerModeOK(getDeviceId());
            //		case IV_FUNCTION_SET_NOTIFY:
            //		case IV_FUNCTION_GET_NOTIFY:
            //		case IV_FUNCTION_GET_SOFTWARE_INFO:
            //		case IV_FUNCTION_GET_FLASH_PAGE:

        default: return 0x0200; // Funktion nicht definiert
        }
    }

    return Channel4ModuleHardware::remoteCall(module, func, data, count);
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::systemReset( short deviceId, unsigned short modus )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, modus );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        count = sizeof(unsigned short);

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Den Resetmodus uebernehmen.
        *(unsigned short*)_packetData = modus;

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM , IV_FUNCTION_SYSTEM_RESET, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Neuaushandlung bei verschluesselter Kommunikation
        getCryptSession().ResetSessionKey();

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, _packetData, count, IV_PENDING_RESET_TRIALS(getTimeout()) )) != IV_ERROR_SUCCESS )
        {
            if ( _result == IV_ERROR_UNREACHABLE )
            {
                _result = IV_ERROR_SUCCESS;
            }
            else
            {
                // Fehler ausgeben
                traceError( getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK );
                return _result;
            }
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }

        // Aktive Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::systemResetOK( short deviceId )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceId );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        trace( "%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__ );	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_RESET_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Leited einen zu übertragenden Datenblock ein. Z. B. Listen.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::beginSetSegment( short deviceId, unsigned short art, unsigned int &address, unsigned int size, unsigned int crc )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %i, %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, art, address, size, crc );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = art;
        count = sizeof(unsigned short);

        // addresse wegen wohlgeformtem Paket mitgeben
        *(unsigned int*)&_packetData[count] = address;
        count += sizeof(unsigned int);

        // geforderte Speichergroesze mitgeben
        *(unsigned int*)&_packetData[count] = size;
        count += sizeof(unsigned int);

        // CRC32 fuer Datenblock mitgeben
        *(unsigned int*)&_packetData[count] = crc;
        count += sizeof(unsigned int);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT );
            return _result;
        }

        // OK

        // Ausgeben der empfangenen Werte

        if ( count == 14 )
        {
            if ( art != *(unsigned short*)_packetData )
            {
                trace( "%s(%i):WARNING(1): %s() [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art );
            }

            address = *(unsigned int*)&_packetData[sizeof(unsigned short)];

            if ( size != *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)] )
            {
                trace( "%s(%i):WARNING(1): %s() [ Size ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)], size );
            }

            if ( crc != *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)*2] )
            {
                trace( "%s(%i):WARNING(1): %s() [ CRC ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)*2], crc );
            }

            trace( "%s(%i):INFO(3): %s() (0x%04X)(0x%08X)(0x%08X)(0x%08X)\n", __FILE__, __LINE__, __func__, art, address, size, crc );
        }
        else 
        {
            trace( "%s(%i):ERROR(1): %s() [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        if ( (art == IV_VALUE_SEGMENT_UPDATE) || (art == IV_VALUE_SEGMENT_UPDATE_BG) || (art == IV_VALUE_SEGMENT_LIST_ACCESS) )
        {
            /*
            short module, func, error = IV_ERROR_SUCCESS;
            // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
            // dieses muss hier seperat abgegriffen werden.
            if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS) {

            Trace("%s(%i):ERROR(1): %s [ Quittierungsprüfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__, __func__);
            return _result;
            }

            if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_BEGIN_SET_SEGMENT_OK)) {

            Trace("%s(%i):INFO(3): %s [ Antwort von CRC OK. ]\n", __FILE__, __LINE__, __func__);
            }

            // evntl. aktiven Fehler auswerten
            if (error != IV_ERROR_SUCCESS) {

            _result = error;
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
            return _result;
            }

            if (*(unsigned short*)_packetData != art) {

            Trace("%s(%i):WARNING(1): %s [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__, __func__);
            } 

            if (count > sizeof(unsigned short)) {

            Trace("%s(%i):WARNING(1): %s [ Datenmenge ist mit (%i) grösser als erwartet. ]\n", __FILE__, __LINE__, __func__, count);
            }
            */
            if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()) )) != IV_ERROR_SUCCESS )
            {
                trace("%s(%i):ERROR(1): %s() [ Quittierungsprüfantwort - CRC schlug fehl. ]\n", __FILE__, __LINE__, __func__);
                return _result;
            }

            if (*(unsigned short*)_packetData != art)
            {
                trace("%s(%i):WARNING(1): %s() [ Die Art der Quittierung ist ungleich. ]\n", __FILE__, __LINE__, __func__);
            } 
        }
    }
    else 
    {
        unsigned char tmp[16];
        count = 0;

        // Adresse wird gesetzt, rest bleibt gleich
        // Meberpuffer aufbereiten
        *(unsigned short*)tmp = art;
        count = sizeof(unsigned short);

        // addresse wegen wohlgeformtem Paket mitgeben
        *(unsigned int*)&tmp[count] = 0;
        count += sizeof(unsigned int);

        // geforderte Speichergroesze mitgeben
        *(unsigned int*)&tmp[count] = size;
        count += sizeof(unsigned int);

        // CRC32 fuer Datenblock mitgeben
        *(unsigned int*)&tmp[count] = crc;
        count += sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }

        count = 0;
        // Aktive Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::beginSetSegmentOK( short deviceId )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceId );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        trace("%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {	
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::endSetSegment(short deviceId, unsigned short art, unsigned int crc)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, art, crc);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        count = sizeof(unsigned short);
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = art;

        // Beendigung einer Uebertragung eines Datanblocks
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT);
            return _result;
        }

        // OK
        if (art != *(unsigned short*)_packetData)
        {
            trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        /*	
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {	
        Trace("%s(%i):ERROR(1): %s [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_END_SET_SEGMENT_OK))
        {			
        Trace("%s(%i):INFO(3): %s [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
        return _result;
        }
        */
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
            return _result;
        }

        if (*(unsigned short*)_packetData != art)
        {
            trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
        }
        else if (*(unsigned int*)&_packetData[sizeof(unsigned short)] != crc)
        {			
            trace("%s(%i):ERROR(1): %s [ CRC ist (0x%08X) soll (0x%08X) ]\n", __FILE__, __LINE__, __func__, *(unsigned int*)&_packetData[sizeof(unsigned short)], crc);
            return -IV_ERROR_CRC_SEGMENT;
        }		
    }
    else
    {
        unsigned char tmp[4];
        count = sizeof(unsigned short);

        // Meberpuffer aufbereiten
        *(unsigned short*)tmp = art;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        // Aktive Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::endSetSegmentOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        count = sizeof(unsigned short);
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK);
            return _result;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_SET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Leited einen zu übertragenden Datenblock ein. Z. B. Listen.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::beginGetSegment(short deviceId, unsigned short art, unsigned int &address, unsigned int &size, unsigned int &crc)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, art, address, size, crc);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        count = sizeof(unsigned short);

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = art;

        // addresse wegen wohlgeformtem Paket mitgeben
        *(unsigned int*)&_packetData[sizeof(unsigned short)] = address;
        count += sizeof(unsigned int);

        // geforderte Speichergroesze mitgeben
        *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)] = size;
        count += sizeof(unsigned int);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT);
            return _result;
        }

        // OK

        // Ausgeben der empfangenen Werte

        if (count == 10)
        {
            if (art != *(unsigned short*)_packetData)
            {
                trace("%s(%i):ERROR(1): %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
            }

            address = *(unsigned int*)&_packetData[sizeof(unsigned short)];
            size    = *(unsigned int*)&_packetData[sizeof(unsigned short) + sizeof(unsigned int)];

            trace("%s(%i):INFO(3): %s [ (0x%04X)(0x%08X)(0x%08X) ]\n", __FILE__, __LINE__, __func__, art, address, size);
        }
        else
        {
            trace("%s(%i):ERROR(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
        /*
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {
        Trace("%s(%i):ERROR(1): %s.Pending [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_BEGIN_SET_SEGMENT_OK))
        {
        Trace("%s(%i):INFO(3): %s.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_SET_SEGMENT_OK);
        return _result;
        }
        */
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK);
            return _result;
        }

        if (count == (sizeof(unsigned short) + sizeof(unsigned int)))
        {
            if (*(unsigned short*)_packetData != art)
            {
                trace("%s(%i):ERROR(1): %s.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
            }

            // CRC uebernehmen
            crc = *(unsigned int*)&_packetData[sizeof(unsigned short)];
        }
        else
        {
            trace("%s(%i):ERROR(1): %s.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }		
    }
    else
    {
        unsigned char tmp[10];
        count = sizeof(unsigned short);

        // Adresse und Size werden zurueckgegeben.

        // Meberpuffer aufbereiten
        *(unsigned short*)tmp = art;

        // addresse wegen wohlgeformtem Paket mitgeben
        *(unsigned int*)&tmp[sizeof(unsigned short)] = address;
        count += sizeof(unsigned int);

        // geforderte Speichergroesze mitgeben
        *(unsigned int*)&tmp[sizeof(unsigned short) + sizeof(unsigned int)] = size;
        count += sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::beginGetSegmentOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_BEGIN_GET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::endGetSegment(short deviceId, unsigned short art)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, art);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        count = sizeof(unsigned short);

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = art;

        // Beendigung einer Uebertragung eines Datanblocks
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT);
            return _result;
        }

        // OK
        if (art != *(unsigned short*)_packetData)
        {
            trace("%s(%i):ERROR: %s [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
        }

        // Bei (den einzelnen innenliegenden) Systemstopps,
        // werden die erfolgreich berechneten CRC - Pruefungen zurueckgemeldet.
        /*
        short module, func, error = IV_ERROR_SUCCESS;
        // nachdem der CRC geprueft wurde, kommt ein Bestaetigungspaket,
        // dieses musz hier seperat abgegriffen werden.
        if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {	
        Trace("%s(%i):ERROR: %s.Pending [ Quittierungsprüfantwort - CRC schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_END_GET_SEGMENT_OK))
        {			
        Trace("%s(%i):INFO(3): %s.Pending [ Antwort von CRC OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK);
        return _result;
        }
        */
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {

            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK);
            return _result;
        }

        if (*(unsigned short*)_packetData != art)
        {			
            trace("%s(%i):ERROR: %s.Pending [ Art ist (%i) soll (%i) ]\n", __FILE__, __LINE__, __func__, *(unsigned short*)_packetData, art);
        }

        if (count != sizeof(unsigned short))
        {		
            trace("%s(%i):ERROR: %s.Pending [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else 
    {
        unsigned char tmp[4];
        count = sizeof(unsigned short);

        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = art;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT, tmp, count)) != IV_ERROR_SUCCESS)
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::endGetSegmentOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);

    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_END_GET_SEGMENT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// Wegen Bootloader nicht vergroesserbar!!
#define IV_FLASH_SIZE_DATA      512

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setFlash(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);
#ifdef _DEBUG
    static int loop = 0;
#endif
    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if (isStreamTransfer()) {

            m_streamBuffer.append((char*)buf, count);

        } else {
            // Gesamten Datenblock aufteilen
            for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
            {
                // Datengroesze 
                sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

                // Memberpuffer aufbereiten
                *(unsigned int*)_packetData = address + addressOffset;
                // Zu setzende Daten uebernehmen
                for (i=0; i < sizeData; i++)
                {
                    _packetData[sizeof(unsigned int) + i] = buf[addressOffset + i];
                }

                // Variable aufbereiten
                sizeData += sizeof(unsigned int);

                if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, _packetData, sizeData)) != IV_ERROR_SUCCESS)
                {
                    // Fehler ausgeben
                    traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH);
                    return _result;
                }

                // Keine Antwortdaten erwartet
                if (sizeData)
                {
                    trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
                }

                //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(unsigned int*)_packetData, *(unsigned short*)&_packetData[sizeof(unsigned int)]);
                /*if (_callBackFunction) {
                if (!_callBackFunction()) {

                _result = -IV_ERROR_BREAK;
                // Fehlernummern setzen
                m_errorCommunicationNum = IV_ERROR_BREAK;
                // Fehlertext ermitteln und eintragen.
                GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

                Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
                }
                } else*/ if (_callBackFunctionWithParams)
                {
                    if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                    {					
                        _result = -IV_ERROR_BREAK;
                        // Fehlernummern setzen
                        _errorCommunicationNum = IV_ERROR_BREAK;
                        // Fehlertext ermitteln und eintragen.
                        getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                        trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                        break;
                    }
                }
            }
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_FLASH, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getFlash(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Angeforderte Laenge eintragen
            *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof(unsigned int) + sizeof(unsigned short);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH);
                return _result;
            }

            if (_callBackFunctionWithParams)
            {
                if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                {					
                    _result = -IV_ERROR_BREAK;
                    // Fehlernummern setzen
                    _errorCommunicationNum = IV_ERROR_BREAK;
                    // Fehlertext ermitteln und eintragen.
                    getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                    trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                    break;
                }
            }

            // OK

            // Empfangene Daten uebernehmen
            for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
            {				
                buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
            }	
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getFlashPhysical(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Angeforderte Laenge eintragen
            *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof(unsigned int) + sizeof(unsigned short);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL);
                return _result;
            }

            if (_callBackFunctionWithParams)
            {
                if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                {					
                    _result = -IV_ERROR_BREAK;
                    // Fehlernummern setzen
                    _errorCommunicationNum = IV_ERROR_BREAK;
                    // Fehlertext ermitteln und eintragen.
                    getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                    trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                    break;
                }
            }

            // OK

            // Empfangene Daten uebernehmen
            for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
            {				
                buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
            }	
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PHYSICAL, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

#define IV_EEPROM_SIZE_DATA   256

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setEEPROM(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? count - addressOffset : IV_EEPROM_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Zu setzende Daten uebernehmen
            for (i=0; i < sizeData; i++)
            {
                _packetData[sizeof(unsigned int) + i] = buf[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof(unsigned int);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM);
                return _result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(unsigned int*)_packetData, *(unsigned short*)&_packetData[sizeof(unsigned int)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if (_callBackFunctionWithParams)
            {
                if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                {					
                    // Fehlernummern setzen
                    _errorCommunicationNum = IV_ERROR_BREAK;
                    // Fehlertext ermitteln und eintragen.
                    getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                    trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                }
            }
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_EEPROM, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getEEPROM(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? count - addressOffset : IV_EEPROM_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Angeforderte Laenge eintragen
            *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof(unsigned int) + sizeof(unsigned short);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM);
                return _result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
            {				
                buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
            }
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_EEPROM, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

    }

    return _result;
}


#define IV_RAM_SIZE_DATA   256

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setRAM(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_RAM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_RAM_SIZE_DATA) > count) ? count - addressOffset : IV_RAM_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Zu setzende Daten uebernehmen
            for (i=0; i < sizeData; i++)
            {
                _packetData[sizeof(unsigned int) + i] = buf[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof(unsigned int);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM);
                return _result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(unsigned int*)_packetData, *(unsigned short*)&_packetData[sizeof(unsigned int)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if (_callBackFunctionWithParams)
            {
                if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                {			
                    // Fehlernummern setzen
                    _errorCommunicationNum = IV_ERROR_BREAK;
                    // Fehlertext ermitteln und eintragen.
                    getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                    trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                }
            }
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RAM, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getRAM(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_EEPROM_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_EEPROM_SIZE_DATA) > count) ? count - addressOffset : IV_RAM_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Angeforderte Laenge eintragen
            *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof(unsigned int) + sizeof(unsigned short);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM);
                return _result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
            {				
                buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
            }
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RAM, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* 2014_10_09(BOt)
* Ab der Version 04.03.04 wird der Parameter count für die Anfrage als in/out verwendet,
* vorher wurden immer Konstant 532 Byte angefordert, der Parameter war lediglich out-Value.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getFlashPage(short deviceId, unsigned short pageNum, unsigned short offs, unsigned char *buf, int &count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, pageNum, buf, count);

    int i;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Meberpuffer aufbereiten
        *(unsigned short*)_packetData = pageNum;
        // Offset in die Seite
        *(unsigned short*)&_packetData[sizeof(unsigned short)] = offs;
        // Angeforderte Laenge eintragen
        *(unsigned short*)&_packetData[sizeof(unsigned short) * 2] = count;

		// Für späteren Vergleich halten.
		int countIn = count;
        // Variable aufbereiten
        count = sizeof(unsigned short) * 3;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE);
            return _result;
        }

        // OK
        if ( countIn != (count - (signed)(sizeof(unsigned short) * 2)) )
        {
            trace("%s(%i):ERROR(1): %s [ Die geforderte Anzahl Bytes wrude nicht geliefert. (in: %i, out: %i) ]\n", __FILE__, __LINE__, __func__, countIn, (count - sizeof(unsigned short) - 2));
        }

        // Empfangene Daten uebernehmen
        for (i=0; i < (count - (signed)(sizeof(unsigned short) * 2)); i++)
        {	
            buf[i] = _packetData[sizeof(unsigned short) * 2 + i];
        }
        count -= sizeof(unsigned short) * 2;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH_PAGE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// Groesse fuer Datenblock
#define IV_MMC_SIZE_DATA      512

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setMMC(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_MMC_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_MMC_SIZE_DATA) > count) ? count - addressOffset : IV_MMC_SIZE_DATA;

            // Memberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Zu setzende Daten uebernehmen
            for (i=0; i < sizeData; i++)
            {
                _packetData[sizeof(unsigned int) + i] = buf[addressOffset + i];
            }

            // Variable aufbereiten
            sizeData += sizeof(unsigned int);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC);
                return _result;
            }

            // Keine Antwortdaten erwartet
            if (sizeData)
            {
                trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, sizeData);
            }

            //Trace("%s(%i):Address: 0x%08X, wrote: 0x%04X Byte\n", __FILE__, __LINE__, __func__, *(unsigned int*)_packetData, *(unsigned short*)&_packetData[sizeof(unsigned int)]);
            /*if (_callBackFunction) {
            if (!_callBackFunction()) {

            _result = -IV_ERROR_BREAK;
            // Fehlernummern setzen
            m_errorCommunicationNum = IV_ERROR_BREAK;
            // Fehlertext ermitteln und eintragen.
            GetErrorCommunication(m_errorCommunicationNum, m_errorCommunication);

            Trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, m_errorCommunication);
            }
            } else*/ if (_callBackFunctionWithParams)
            {
                if (!_callBackFunctionWithParams(count, addressOffset + sizeData))
                {
                    _result = -IV_ERROR_BREAK;
                    // Fehlernummern setzen
                    _errorCommunicationNum = IV_ERROR_BREAK;
                    // Fehlertext ermitteln und eintragen.
                    getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                    trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                    break;
                }
            }
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_MMC, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getMMC(short deviceId, unsigned int address, unsigned char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, address, buf, count);

    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Gesamten Datenblock aufteilen
        for (addressOffset=0; addressOffset < count; addressOffset += IV_MMC_SIZE_DATA)
        {
            // Datengroesze 
            sizeData = ((addressOffset + IV_MMC_SIZE_DATA) > count) ? count - addressOffset : IV_MMC_SIZE_DATA;

            // Meberpuffer aufbereiten
            *(unsigned int*)_packetData = address + addressOffset;
            // Angeforderte Laenge eintragen
            *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

            // Variable aufbereiten
            sizeData = sizeof(unsigned int) + sizeof(unsigned short);

            if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC, _packetData, sizeData)) != IV_ERROR_SUCCESS)
            {
                // Fehler ausgeben
                traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC);
                return _result;
            }

            // OK

            // Empfangene Daten uebernehmen
            for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
            {
                buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
            }
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_MMC, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Struktur zum SFS.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param serial Variable die die ermittelte Seriennummer erhaehlt.
*/
int Channel4ModuleSystem::getSFSMPB(short deviceId, unsigned char &type, SFSMPB &sfs)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count != (sizeof(unsigned char) + sizeof(SFSMPB)))
        {
            trace("%s(%i):WARNING(1): %s [ SFS ungleich der erwarteten Größe ]\n", __FILE__, __LINE__, __func__);
            type = 0;
            memset(&sfs, 0, sizeof(SFSMPB));
        }
        else
        {
            type = *_packetData;
            sfs = *(SFSMPB*)&_packetData[1];
        }
    }
    else
    {
        unsigned char tmp[64];
        count = sizeof(unsigned char) + sizeof(SFSMPB);
        memset(tmp, 0, sizeof(tmp));

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SFSMPB, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Firmwareversion.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param version Zeiger Variable fuer den Versionsstring.
*                Format: XX.XX.XX.XX ?...
*/
int Channel4ModuleSystem::getVersion(short deviceId, char *version)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, version);

    int i, count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {		
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION);
            return _result;
        }

        // OK
        for (i=0; version && i < count; i++)
        {			
            version[i] = _packetData[i];
        }
    }
    else
    {
        unsigned char tmp[64];
        count = sprintf((char*)tmp, "04.02.03.02.DLL") + 1;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION, tmp, count)) != IV_ERROR_SUCCESS)
        {		
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Bootloaderversion.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param version Zeiger auf Variable fuer Versionsstring. 
*                Format: XX.XX.XX.XX ?...
*/
int Channel4ModuleSystem::getVersionBootloader(short deviceId, char *version)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, version);

    int i, count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER);
            return _result;
        }

        // OK
        for (i=0; i < count; i++)
        {
            version[i] = _packetData[i];
        }
    }
    else
    {
        unsigned char tmp[64];
        count = sprintf((char*)tmp, "04.01.04.99") + 1;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_VERSION_BOOTLOADER, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Seriennummer des Geraets.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param serial Variable die die ermittelte Seriennummer erhaehlt.
*/
int Channel4ModuleSystem::getSerial(short deviceId, unsigned int &serial)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, serial);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count > (int)sizeof(unsigned int))
        {		
            trace("%s(%i):WARNING(1): %s [ Seriennummer mit %i Byte, laenger als erwartete 32bit ]\n", __FILE__, __LINE__, __func__, count - 4);
        }

        serial = *(unsigned int*)_packetData;
    }
    else
    {		
        unsigned char tmp[64];
        count = sizeof(unsigned int);
        *(unsigned int*)tmp = _serial;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SERIAL, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Setzten der RTC (RealTimeClock) im Geraet.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param time Zeitstring im Format: "JJJJ-MM-DD hh:mm:ss"
*/
int Channel4ModuleSystem::setRTC(short deviceId, char *time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, time);

    int i, count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        count = 20;
        // Puffer loeschen
        memset(_packetData, 0, sizeof(_packetData));
        for (i=0; i < (count - 1); i++)
        {
            _packetData[i] = time[i];
        }

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }	

    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RTC, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Lesen der RTC (RealTimeClock) aus Geraet.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param time Zeiger auf Zeichenkette fuer Zeitstring. 
*             Format: "JJJJ-MM-DD hh:mm:ss"
*/
int Channel4ModuleSystem::getRTC(short deviceId, char *time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, time);

    int i, count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count > 20)
        {
            trace("%s(%i):WARNING(1): %s [ Uhrzeitstring ist mit %i Byte laenger als erwartet. ]\n", __FILE__, __LINE__, __func__, count - 20);
        }

        // OK
        for (i=0; i < count; i++)
        {
            time[i] = _packetData[i];
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RTC, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

unsigned char Channel4ModuleSystem::hexToUINT8(char *data)
{
    int i;
    unsigned char val = 0;

    for (i=0; i < 2; i++)
    {
        val = (val << 4);

        if (data[i] >= '0' && data[i] <= '9')
        {
            val += data[i] - '0';
        }
        else if (data[i] >= 'a' && data[i] <='f')
        {
            val += data[i] - 'a' + 10;
        }
        else
        {
            val += data[i] - 'A' + 10;
        }
    }

    return val;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setSystemVariable( short deviceId, char *name, char *val )
{
    trace( "%s(%i):INFO(3): %s [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, name, val );

    int i;
    int j;
    int count = 0;
    char lName[100];
    char lValue[IV_PACKET_LENGTH_DATA];

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Wert für ggf. Anpassungen übernehmen
        memset( lValue, 0, sizeof(lValue) );
        strcpy( lValue, val );

        // Name für Vergleich übernehmen
        memset( lName, 0, sizeof(lName) );
        strcpy( lName, name );
        for ( i = 0; i < (int)strlen( lName ); ++i )
        {
            lName[i] = (char)toupper(lName[i]);
        }

        if ( (strstr( lName, "MOBILE" ) == lName || strstr( lName, "_MODEM_MC35I" ) == lName) && strstr( lName, "HTTPSEND" ) == NULL )
        {
            char version[64];            
            memset( version, 0, sizeof(version) );
            if ( (_result = getVersion( deviceId, version )) != IV_ERROR_SUCCESS )
            {
                return _result;
            }

            // Format prüfen
            if (isxdigit(version[0]) &&
                isxdigit(version[1]) &&
                version[2] == '.' &&
                isxdigit(version[3]) &&
                isxdigit(version[4]) &&
                version[5] == '.' && 
                isxdigit(version[6]) &&
                isxdigit(version[7]) &&
                version[8] == '.' &&
                isxdigit(version[9]) &&
                isxdigit(version[10]) &&
                version[11] == '.')
            {
                unsigned int ver = (hexToUINT8(version) << 24);
                ver += (hexToUINT8(&version[3]) << 16);
                ver += (hexToUINT8(&version[6]) << 8);
                ver += hexToUINT8(&version[9]);

                if (ver >= 0x04010600)
                {
                    mobileMaskedSysvar(lValue);
                }
            }
        }

        // Puffer loeschen
        memset(_packetData, 0, sizeof(_packetData));
        // Namen uebernehmen
        count = (int)strlen(name) + 1;
        for (i=0; i < count; i++)
        {
            _packetData[i] = name[i];
        }
        // Wert uebernehmen
        count += (int)strlen(lValue) + 1;
        for (j=i; i < count; i++)
        {
            _packetData[i] = lValue[i - j];
        }

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getSystemVariable(short deviceId, const char *name, char *val)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, name, val);

    int i, count = 0;
    char lName[100];
    char lValue[IV_PACKET_LENGTH_DATA]; // Größe aufgrund von SYSTEM.STATUS nötig.

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Puffer loeschen
        memset(_packetData, 0, sizeof(_packetData));
        // Namen uebernehmen
        count = (int)strlen(name) + 1;
        for (i=0; i < count; i++)
        {
            _packetData[i] = name[i];
        }

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count > 80)
        {
            trace("%s(%i):WARNING(3): %s [ Laenge der Variablen mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Namen uebergehen
        count = (int)strlen(name) + 1;

        // Wert fuer ggf. Anpassungen übernehmen
        memset(lValue, 0, sizeof(lValue));

        // Wert uebernehmen
        strncpy(lValue, (char*)&_packetData[count], sizeof(lValue) - 1);	

        // Name für Vergleich übernehmen
        memset(lName, 0, sizeof(lName));
        strcpy(lName, name);
        for (i=0; i < (int)strlen(lName); i++)
        {
            lName[i] = (char)toupper(lName[i]);
        }
        if ((strstr(lName, "MOBILE") == lName || strstr(lName, "_MODEM_MC35I") == lName) && strstr(lName, "HTTPSEND") == NULL)
        {
            char version[64];            
            memset( version, 0, sizeof(version) );
            if ( (_result = getVersion( deviceId, version )) != IV_ERROR_SUCCESS )
            {
                return _result;
            }

            // Format prüfen
            if (isxdigit(version[0]) &&
                isxdigit(version[1]) &&
                version[2] == '.' &&
                isxdigit(version[3]) &&
                isxdigit(version[4]) &&
                version[5] == '.' && 
                isxdigit(version[6]) &&
                isxdigit(version[7]) &&
                version[8] == '.' &&
                isxdigit(version[9]) &&
                isxdigit(version[10]) &&
                version[11] == '.')
            {
                unsigned int ver = (hexToUINT8(version) << 24);
                ver += (hexToUINT8(&version[3]) << 16);
                ver += (hexToUINT8(&version[6]) << 8);
                ver += hexToUINT8(&version[9]);

                if (ver >= 0x04010600)
                {
                    mobileUnmaskedSysvar(lValue);
                }
            }
        }

        // Wert uebernehmen
        strcpy(val, lValue);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }	
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setRelay(short deviceId, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, name, busNum, relayNum, status, time);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Parameter uebernehmen
        count = sprintf((char*)_packetData, "%s", name) + 1;

        // Weitere Parameter als String uebergehen
        count += sprintf((char*)&_packetData[count], "%i", busNum) + 1;
        count += sprintf((char*)&_packetData[count], "%i", relayNum) + 1;
        count += sprintf((char*)&_packetData[count], "%i", status) + 1;
        count += sprintf((char*)&_packetData[count], "%i", time) + 1;
        /*
        // Busnummer
        _packetData[count] = busNum;
        count += sizeof(unsigned char);

        // Relaisnummer
        _packetData[count] = relayNum;
        count += sizeof(unsigned char);

        // Status
        _packetData[count] = status;
        count += sizeof(unsigned char);

        // Time
        *(unsigned short*)&_packetData[count] = status;
        count += sizeof(unsigned short);
        */

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_RELAY, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getRelay(short deviceId, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, name, busNum, relayNum, status, time);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Namen uebernehmen
        count = sprintf((char*)_packetData, "%s", name) + 1;

        // Weitere Parameter als String uebernehmen
        count += sprintf((char*)&_packetData[count], "%i", busNum) + 1;
        count += sprintf((char*)&_packetData[count], "%i", relayNum) + 1;
        /*
        // Busnummer
        _packetData[count] = busNum;
        count += sizeof(unsigned char);

        // Relaisnummer
        _packetData[count] = relayNum;
        count += sizeof(unsigned char);
        */	
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY);
            return _result;
        }

        // Name
        count = (int)strlen((char*)_packetData) + 1;

        // Weitere Parameter als String uebergehen
        // busNum, relayNum
        count += (int)strlen((char*)&_packetData[count]) + 1;
        count += (int)strlen((char*)&_packetData[count]) + 1;

        // Status uebernehmen
        status = atoi((char*)&_packetData[count]);
        count += (int)strlen((char*)&_packetData[count]) + 1;

        // Zeit uebernehmen
        time = atoi((char*)&_packetData[count]);
        /*
        // Keine Antwortdaten erwartet
        if (count != (sizeof(unsigned char) + sizeof(unsigned short))) {

        Trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Status
        status = _packetData[0];
        // Time
        time = *(unsigned short*)&_packetData[sizeof(unsigned char)];
        */	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_RELAY, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterBios(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, _packetData, count, 1)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
        /*	
        // Nach dem Wechsel ins Bios wird eine Bestaetigung gesendet.
        short module, func, error = IV_ERROR_SUCCESS;
        if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
        Trace("%s(%i): %s() Quittierungsantwort - schlug fehl.\n", __FILE__, __LINE__, __func__);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_ENTERED_BIOS))
        {			
        Trace("%s(%i): %s() Antwort OK.\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS);
        return _result;
        }
        */
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS);
            return _result;
        }		
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enteredBios(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTERED_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::exitBios(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Beendigung einer Uebertragung eines Datanblocks
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        /*	
        // Nach dem Verlassen des Bios kommt eine Bestaetigungsanfrage.
        short module, func, error = IV_ERROR_SUCCESS;
        if ((_result = Pending(deviceId, module, func, error, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
        Trace("%s(%i):ERROR(1): %s [ Quittierungsantwort - schlug fehl (%i) ]\n", __FILE__, __LINE__, __func__, _result);
        return _result;
        }

        if ((module == IV_MODULE_SYSTEM) && (func == IV_FUNCTION_EXITED_BIOS))
        {			
        Trace("%s(%i):INFO(3): %s [ Antwort OK ]\n", __FILE__, __LINE__, __func__);
        }

        // evntl. aktiven Fehler auswerten
        if (error != IV_ERROR_SUCCESS)
        {
        _result = error;
        // Fehler ausgeben
        traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS);
        return _result;
        }
        */
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS);
            return _result;
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::exitedBios(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXITED_BIOS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setInterface(short deviceId, unsigned char type)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Den Geraetetyp uebernehmen.
        _packetData[0] = type;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_INTERFACE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getInterface(short deviceId, unsigned char &type)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE);
            return _result;
        }

        // OK
        if (count != sizeof(unsigned char))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        type = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INTERFACE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setBaud(short deviceId, unsigned int br)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, br);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Den Geraetetyp uebernehmen.
        *(unsigned int*)&_packetData[0] = br;
        count += sizeof(unsigned int);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_BAUD, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getBaudRate(short deviceId, unsigned int &br)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, br);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD);
            return _result;
        }

        // OK
        if (count != sizeof(unsigned int))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        br = *(unsigned int*)_packetData;
        switch (br)
        { // Bereich pruefen
        case 9600:
        case 14400:
        case 19200:
        case 38400:
        case 57600:
        case 115200:
            break;

        default:
            br = 38400;
        }	

    }
    else 
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_BAUD, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setDeviceIdBios(short deviceId, unsigned short id)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, id);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Den Geraetetyp uebernehmen.
        *(unsigned short*)&_packetData[0] = id;
        count += sizeof(unsigned short);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_DEVICE_ID, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getDeviceIdBios(short deviceId, unsigned short &id)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, id);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID);
            return _result;
        }

        // OK
        if (count != sizeof(unsigned short))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        id = *(unsigned short*)_packetData;
        if ( id < 1 || id > 254 )
            id = 254;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_DEVICE_ID, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterBootloader(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, _packetData, count, 1)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_BOOTLOADER, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterXPortMonitor(short deviceId, char *data, int &count)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, data);

    int lCount = 0, i;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, _packetData, lCount)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (lCount)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK);
            return _result;
        }

        // Daten uebernehmen
        for (i=0; i < count; i++)
        {			
            data[i] = _packetData[i];
        }		
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterXPortMonitorExt(short deviceId, char *data, int &count)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, data);

    int lCount = 0, i;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT, _packetData, lCount)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (lCount)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK);
            return _result;
        }

        // Daten uebernehmen
        for (i=0; i < count; i++)
        {
            data[i] = _packetData[i];
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterXPortMonitorOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::enterXPortMonitorExtOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_ENTER_XPORT_MONITOR_EXT_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::commandXPort(short deviceId, char *buf, int &count)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, buf, count);

    int i;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Daten fuer senden uebernehmen
        for (i=0; i < count; i++)
        {
            _packetData[i] = buf[i];
        }

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT);
            return _result;
        }

        // OK

        // Daten von XPort uebernehmen
        for (i=0; i < count; i++)
        {
            buf[i] = _packetData[i];
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_CMD_XPORT, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::exitXPortMonitor(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Beendigung einer Uebertragung eines Datanblocks
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }


        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK);
            return _result;
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::exitXPortMonitorOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_EXIT_XPORT_MONITOR_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::openComServerMode(short deviceId, char *comm, unsigned int baudRate, unsigned char byteSize, unsigned char parity, unsigned char stopBits, unsigned short flags, unsigned short timeoutToClose)
{
    trace("%s(%i):INFO(3): %s [ %i, %s, %i, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, comm, baudRate, byteSize, parity, stopBits, timeoutToClose);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Alle Zusatzparameter uebernehmen
        count += sprintf((char*)&_packetData[count], "%s", comm) + 1;
        *(unsigned int*)&_packetData[count] = baudRate;
        count += sizeof(unsigned int);
        _packetData[count] = byteSize;
        count += sizeof(unsigned char);
        _packetData[count] = parity;
        count += sizeof(unsigned char);
        _packetData[count] = stopBits;
        count += sizeof(unsigned char);
        *(unsigned short*)&_packetData[count] = flags;
        count += sizeof(unsigned short);
        *(unsigned short*)&_packetData[count] = timeoutToClose;
        count += sizeof(unsigned short);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Aktive Antwort abwarten
        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(getDeviceIdRemote(), IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK);
            return _result;
        }		
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::openComServerModeOK(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        trace("%s(%i):ERROR(1): %s", __FILE__, __LINE__, __func__);
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_OPEN_COM_SERVER_MODE_OK, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}
// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::closeComServerMode(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_CLOSE_COM_SERVER_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Information zu einem Modultyp
*
* @param deviceId Anzusprechende Geraetenummer.
* @param name Das anzusprechende Modul. (ACCESS.MODULE fuer Zutrittskontrollmodul)
* @param param Abhaengig vom verwendeten Namen. (Bei ACCESS.MODULE gibt es die TM-Nummer des Moduls )
* @param info Zeiger auf Zeichenkettenpuffer fuer Rueckgabe.
* @param infoLength Laenge des Zeichenkettenpuffers fuer Rueckgabe
*/
int Channel4ModuleSystem::getInfo(short deviceId, const char *name, const char *param, char *info, int &infoLength)

{
    trace("%s(%i):INFO(3): %s [ %i, %s, %s, %s, %i ]\n", __FILE__, __LINE__, __func__, deviceId, name, param, info, infoLength);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        std::string name_( name );
        std::string param_( param );

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);
 
        // Ab der Version 04.03.04 kann bei ACCESS.MODULE bei param noch mit CR die Bus-Id angefügt werden,
        // Vorgängerversionen verstehen diese Angabe nicht und sie muss daher ersetzt werden.
        if ( name_ == "ACCESS.MODULE" )
        {
            int res;
            std::string regTM("[0-3]\\d[01](\\r\\d{1,4})?");

            // Format pruefen
            if ( !std::regex_match( param_, std::regex( regTM ) ) )
                return -IV_ERROR_FUNCTION_PARAMETER;

            if ( param_.find_first_of( '\r' ) != std::string::npos )
            {
                std::array<char, IV_PACKET_LENGTH_DATA> buffer;

                // Format pruefen
                if ( !std::regex_match( param_, std::regex( regTM ) ) )
                    return -IV_ERROR_FUNCTION_PARAMETER;

                // Firmwareversion ermitteln.
                if ( (res = getVersion( deviceId, buffer.data() )) != IV_ERROR_SUCCESS )
                    return res;

                CDFSoftwareVersion version( buffer.data() );
                // Kleiner dieser Version muss zusätzlich geprüft werden.
                if ( version < "04.03.04" )
                {
                    // Aufteilen des Strings in seine Bestandteile.
                    std::vector<std::string> tokens;
                    boost::split( tokens, param_, boost::is_any_of( "\r" ) );
                    if ( tokens.size() < 2 )
                        return -IV_ERROR_FUNCTION_PARAMETER;

                    // Angeggebene MasterId gegen die eingestellte prüfen.

                    // Systemvariable ACCESS.MASTERID ermitteln.
                    if ( (res = getSystemVariable( deviceId, "ACCESS.MASTERID", buffer.data() )) != IV_ERROR_SUCCESS )
                        return res;

                    std::string masterId( buffer.data() );
                    // Prüfen ob die richtige MasterId angesprochen werden soll.
                    if ( std::stoi( masterId ) != std::stoi( tokens[1] ) )
                        return -IV_ERROR_FUNCTION_PARAMETER;

                    // Terminierung hinter der Türmodulangabe.
                    param_[tokens[0].size()] = '\0';
                }
            }
        }

        count = strlen( strcpy( (char*)_packetData, name_.data() ) ) + 1;
        count += strlen( strcpy( (char*)&_packetData[count], param_.data() ) ) + 1;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO);
            return _result;
        }

        // Antwortdaten erwartet
        if ( (unsigned)count < (name_.size() + 1 + param_.size() + 1) )
        {
            trace("%s(%i):WARNING(1): %s [ Antwort ist zu kurz. ]\n", __FILE__, __LINE__, __func__);
        }

        int i, offs;
        // Name uebergehen
        offs = (int)strlen((char*)_packetData) + 1;
        // Parameter uebergehen
        offs += (int)strlen((char*)&_packetData[offs]) + 1;

        // Wert uebernehmen
        for (i=0; (i < (count - offs)) && (i < infoLength); i++)
        {		
            info[i] = _packetData[offs + i];
        }
        // Uebergebene Laenge mitteilen
        infoLength = i - 1;		
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_INFO, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param type Benachrichtigungstyp
* @param state Zu setzender Zustand.
*/
int Channel4ModuleSystem::setNotify(short deviceId, unsigned char type, unsigned char state)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type, state);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        _packetData[count] = type;
        count += sizeof(unsigned char);
        _packetData[count] = state;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}


// ##################################################################
/**
* @brief Aktivieren oder Deaktivieren einer Benachrichtigung.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param type Benachrichtigungstyp
* @param state Aktuell gesetzter Zustand.
*/
int Channel4ModuleSystem::getNotify(short deviceId, unsigned char type, unsigned char &state)

{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type, state);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        _packetData[count] = type;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY);
            return _result;
        }

        // Antwortdaten erwartet
        if (count != sizeof(unsigned char))
        {
            trace("%s(%i):WARNING(1): %s [ Antwort ist zu lang oder kurz. ]\n", __FILE__, __LINE__, __func__);
        }
        else
        {
            // Gemeldeten Zustand uebernehmen
            state = _packetData[0];
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Abfragen der Softwareinfo der eingespielten Anwendungsfirmware.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param swi Puffer für die Softwareinfo.
* @param infoLength Länge des Puffers und übergebene Anzahl daten.
*/
int Channel4ModuleSystem::getSoftwareInfo(short deviceId, unsigned char *swi, int &infoLength)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, swi, infoLength);

    int i, count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO);
            return _result;
        }

        // Antwortdaten erwartet
        for (i=0; i < count && i < infoLength; i++)
        {
            swi[i] = _packetData[i];
        }
        infoLength = i;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SOFTWARE_INFO, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceIDRemote Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setSystemFlags(short deviceId, unsigned int flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, flags);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);


        // Den Geraetetyp uebernehmen.
        *(unsigned int*)&_packetData[0] = flags;
        count += sizeof(unsigned int);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {			
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {		
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYSTEM_FLAGS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceIDRemote Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getSystemFlags(short deviceId, unsigned int &flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, flags);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS, _packetData, count)) != IV_ERROR_SUCCESS)
        {			
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS);
            return _result;
        }

        // OK
        if (count != sizeof(unsigned int))
        {			
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        flags = *(unsigned int*)_packetData;
    }
    else
    {		
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYSTEM_FLAGS, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }	
    }

    return _result;
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::systemReadRecord(short deviceId, unsigned char *buf, int &count, unsigned char &status)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x08X, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, buf, count, status);

    int i;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD);
            return _result;
        }

        // OK
        switch (_packetData[0])
        {
        case IV_ANSW_NO_SYS_RECORD:         // kein Datensatz vorhanden
            status = 0;
            break;

        case IV_ANSW_ONE_SYS_RECORD:        // ein Systemdatensatz 
            status = 1;
            break;

        default:
            status = 0;
            break;
        }

        if (((status == 0) && (count == 0)) || (count > IV_SYS_RECORDS_MAX))
        {
            return -IV_ERROR_PROCESSING_DLL;
        }

        for (i=0; i < count; i++)
        {
            buf[i] = _packetData[i];
        }
    }
    else
    {
        unsigned char data[IV_SYS_RECORDS_MAX];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_SYS_RECORD, data, count)) != IV_ERROR_SUCCESS) {

            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

int Channel4ModuleSystem::systemQuitRecord(short deviceId, unsigned char &status)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, status);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD);
            return _result;
        }

        // OK
        if (count > (int)sizeof(unsigned char))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        switch (_packetData[0])
        {
        case 0:
            status = 0;
            break;

        case IV_ANSW_ACK_ON_SYS_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
            status = 1;
            break;
        }
    }
    else 
    {
        unsigned char data[1];

        ////if (m_records.size()) {

        ////	CRecord &record = m_records.front();

        ////	switch(record._packetData[0]) {
        ////	case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
        ////		data[0] = 0;
        ////		break;
        ////	case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
        ////	case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
        ////		data[0] = MB_ANSW_ACK_ON_RECORD;
        ////		break;
        ////	case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
        ////	case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
        ////		data[0] = MB_ANSW_ACK_OFF_RECORD;
        ////		break;
        ////	}

        ////	count = sizeof(unsigned char);

        ////	// Datensatz loeschen
        ////	m_records.pop();
        ////} else {

        ////	data[0] = 0;
        ////	count = sizeof(unsigned char);
        ////}

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_SYS_RECORD, data, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;

}

int Channel4ModuleSystem::systemRestoreRecords(short deviceId)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        //		while (m_records.size()) {
        //			m_records.pop();
        //		}

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_RESTORE_ALLSYS_RECORDS, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;

}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::passwordCheck(short deviceId, char *password)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, password);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Ob der Befehl unterstützt wird und das Gerät geschützt ist, intern halten.
        setFlagPasswordProtection(FALSE);
        setFlagPasswordProtected(FALSE);

        // Puffer loeschen
        memset(_packetData, 0, sizeof(_packetData));
        // Der Passwortpuffer wird mit 17 Zeichen verschluesselt
        count = 17;

        // Initialisieren der Verschluesselung
        df4LoadCryptKey();
        df4Init(TRUE);

        // Nur wenn Passwort angegeben wurde.
        if (strlen(password)) {

            // Passwort uebernehmen
            strncpy((char*)_packetData, password, 16);
        }

        // Verschluesseln
        df4(_packetData, count);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK);

            // Fehler auswerten
            if (((_result >> 8) & 0xFF) == 2)
            {
                // Funktion unbekannt
            }
            else if ((_result & 0xFF) == 1)
            {
                // Funktionsfehler, passwort falsch
                // Passwortschutz ist möglich.
                setFlagPasswordProtection(TRUE);
            }

            return _result;
        }

        // Befehl erfolgreich, also wird Passwortschutz voll genutzt.
        setFlagPasswordProtection(TRUE);
        setFlagPasswordProtected(TRUE);

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_CHECK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::passwordKey(short deviceId, char *key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, key);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Keine Daten zu senden
        count = 0;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY);
            return _result;
        }

        // n Antwortdaten erwartet
        if (count != 17)
        {
            trace("%s(%i):WARNING(1): %s [ Laenge des Passwortschlüssels mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert uebernehmen
        strcpy(key, (char*)_packetData);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PASSWORD_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::pressVirtualKey(short deviceId, unsigned char vk, unsigned char flags)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, vk, flags);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Keycode
        _packetData[count] = vk;
        count += sizeof(unsigned char);

        // Flags
        _packetData[count] = flags;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_PRESS_VIRTUAL_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::writeStream(short deviceId, streamType type, const char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type, buf, count);
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    { 
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Befehlsparamter aufbereiten.
        sizeData = 0;
        // Type der Streamdaten
        *(unsigned short*)_packetData = type;
        sizeData += sizeof(unsigned short);
        // Offset falls der Stream wegen CRC-Fehler ab einer bestimmten Position neu gesendet werden soll. (Vorgesehen)
        *(unsigned int*)&_packetData[sizeData] = 0;
        sizeData += sizeof(unsigned int);
        // Anzahl Datenbyte des Streams
        if (buf) {

            *(unsigned int*)&_packetData[sizeData] = count;
        } else {

            *(unsigned int*)&_packetData[sizeData] = 0;
        }
        sizeData += sizeof(unsigned int);
        // Stream-CRC
        if (buf) {

            *(unsigned int*)&_packetData[sizeData] = GetCRC32(reinterpret_cast<unsigned char*>(const_cast<char*>(buf)), count);
        } else {

            *(unsigned int*)&_packetData[sizeData] = 0;
        }
        sizeData += sizeof(unsigned int);
        // Vorgesehenes Flagbyte fuer unterstuetzung des Blockweisen Streamings, und ggf. Erweiterungen.
        _packetData[sizeData] = 0;
        sizeData += sizeof(unsigned char);

        // Grundbefehl vor Sendung des Streams ausfuehren.
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM);
            return _result;
        }

        // Liegt ein Stream vor?
        if (!buf)
            return _result;

        // Ggf. Worker im Active-Mode anhalten.
        suspend();

        char *bufBase = const_cast<char*>(buf);
        char *pos = bufBase;
        int bufCount = count;

        // Daten ggf. verschluesseln.
        std::string outCrypt;
        if (isCrypt()) {

            outCrypt.assign(buf, count);
            writeStreamCrypt(outCrypt);

            bufBase = const_cast<char*>(outCrypt.data());
            pos = bufBase;
            bufCount = outCrypt.size();
        }

        // Zu versendendes Stream-Protokoll-Paket erstellen.
        unsigned int uiTmp = bufCount + sizeof(unsigned short);
        unsigned short usTmp = 0;
        std::string prot;
        // Brutto Laenge ( 32bit ) eintragen.
        prot += (char)uiTmp & 0xFF;
        prot += (char)(uiTmp >> 8) & 0xFF;
        prot += (char)(uiTmp >> 16) & 0xFF;
        prot += (char)(uiTmp >> 24) & 0xFF;
        // Reserve ( 16bit ) eintragen.
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;
        // Kopf-CRC berechnen und eintragen.
        usTmp = GetCRC16((unsigned char*)prot.data(), prot.size());
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;
        // Daten uebernehmen
        prot.append(pos, bufCount);
        // Daten-CRC berechnen und eintragen.
        usTmp = GetCRC16((unsigned char*)pos, bufCount);
        prot += (char)usTmp & 0xFF;
        prot += (char)(usTmp >> 8) & 0xFF;

        // Zeiger und Laengenangabe auf das Stream-Protokoll-Paket setzen.
        bufBase = const_cast<char*>(prot.data());
        pos = bufBase;
        bufCount = prot.size();

        // Beim Kopieren des Stream-Protokoll-Paket werden alle 0x7E und 0x7F Bytes maskiert.
        std::string out;
        for (; (pos - bufBase) < bufCount; pos++) {

            if (*pos == 0x7E) {

                out += (char)0x7F;
                out += (char)0x01;                    
            } else if (*pos == 0x7F) {

                out += (char)0x7F;
                out += (char)0x00;
            } else {

                out += *pos;
            }
        }

        int transmittedCount = 0;
        // Stream-Protokoll-Paket senden.
        if ((_result = writeData(out.data(), out.size(), transmittedCount, IV_WRITE_TRIALS)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSMIT_STREAM);
            // Ggf. Worker im Active-Mode wieder starten.
            resume();
            return _result;
        }

        if (sizeData >= 2) {

            int res = IV_ERROR_SUCCESS, receiveCount, receivedCount;
            bool masked;

            unsigned int crc = -1;
            char *crcPos = const_cast<char*>(buf);
            unsigned int crcChunk = 0;
            int statusCount = 1;

            // In der Antwort werden folgende Werte geliefert:
            // - 8bit Flagbyte (reserviert)
            // - 8bit Die zu erwartende Groesse fuer die Status-Meldungen, in Byte.
            // - 32bit Maximale Puffergroesse, fuer Blockweises Streaming.

            unsigned char statusSize = _packetData[1];
            if (isCrypt()) {

                // Wenn die Daten verschluesselt werden, dann kommt es zu einer Datenmengenerhoehung
                // aufgrund der 16 Byte Bloecke beim AES.
                // Zusaetzlich werden die Daten um einen CRC und Anzahl Nutzdaten erweitert, daher (count + 5)!
                unsigned int count_blocks = (statusSize + 5) / 16;
                if ((statusSize + 5) % 16) count_blocks++;

                statusSize = count_blocks * 16;
            }

            // Paketframe noch dazurechnen
            statusSize += 10;

            // Aktuell werden folgende Werte im Status geliefert:
            // - 32bit bereits verarbeitete Anzahl Byte
            // - 32bit crc zu dieser Verarbeitungsposition
            // Speicher fuer Status-Meldung.
            boost::scoped_array<unsigned char> statusReceive(new unsigned char[statusSize]);
            unsigned char *statusReceivePos;
            boost::scoped_array<unsigned char> status(new unsigned char[statusSize]);
            unsigned char *statusPos;

            // In einer Schleife so lange die Statusmeldungen abrufen,
            // bis die Information vorliegt, dass alle Daten verarbeitet wurden.
            do {
                receiveCount = statusSize;
                receivedCount = 0;
                masked = false;
                // Annahme beginnt am Anfang des Puffers.
                statusPos = status.get();

                // Es wird maximal 25 Sekunden zwischen den einzelnen Statusmeldungen gewartet.
                int retries = IV_PENDING_TRIALS(getTimeout());
                for (; retries >= 0; retries--) {

                    // Leseschleife um Maskierte Zeichen herausfiltern.
                    while (receiveCount && (res = readData(statusReceive.get(), receiveCount, receivedCount, 1)) == IV_ERROR_SUCCESS) {

                        // Da nun Zeichen empfangen wurden wird nicht weiter auf den Lesebeginn gewartet.
                        retries = 0;

                        // Fuer jedes maskierte Zeichen muss nachher eins nachgelesen werden.
                        receiveCount = 0;

                        // Auswertung beginnt am Anfang des Temp Puffers.
                        statusReceivePos = statusReceive.get();

                        // Gelesene Daten in Status uebernehmen
                        for (;receivedCount; receivedCount--) {

                            // Aktuelles Zeichen "maskiert" ?
                            if (masked) {

                                switch (*statusReceivePos) {
                                case 0x00:
                                    *statusPos = 0x7F;
                                    break;
                                case 0x01:
                                    *statusPos = 0x7E;
                                    break;
                                default:
                                    // Zeichen bleibt unveraendert.
                                    *statusPos = *statusReceivePos;
                                    break;
                                }
                                // Flag fuer "maskiert" zurucknehmen.
                                masked = false;

                                // Naechstes Zeichen
                                statusPos++;

                            } else if (*statusReceivePos == 0x7F) {

                                // Flag fuer "maskiert" setzen. In der naechsten Runde mit seiner Entsprechung ersetzen.
                                masked = true;
                                // Fuer dieses "maskiert" ist ein nachlesen erforderlich.
                                receiveCount++;

                            } else {

                                // Unmaskiertes Zeichen uebernehmen.
                                *statusPos = *statusReceivePos;
                                // Naechstes Zeichen
                                statusPos++;
                            }

                            // Pufferueberlauf vermeiden.
                            if ((statusPos - status.get()) > statusSize)
                                break;

                            // Auf naechstes Zeichen fuer Uebernahme wechseln.
                            statusReceivePos++;
                        }
                    }
                }

                // Keine Statusmeldung im vorgegeben Zeitraum?
                if (retries == 0) {

                    trace("%s(%i):ERROR(1): %s [ status timeout ]\n", __FILE__, __LINE__, __func__);
                    // Reservierten Speicher freigeben.
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_NO_REQUEST;                      
                }

                if (res != IV_ERROR_SUCCESS) {

                    trace("%s(%i):ERROR(1): %s [ ReceiveData fail %u ]\n", __FILE__, __LINE__, __func__, res);
                    // Reservierten Speicher freigeben.
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return res;                      
                }

                // Wurde die erwartete Anzahl Zeichen empfangen?
                if ((statusPos - status.get()) != statusSize) {

                    trace("%s(%i):ERROR(1): %s [ status size %u < %u ]\n", __FILE__, __LINE__, __func__, (statusPos - status.get()), statusSize);
                    // Reservierten Speicher freigeben.
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_UNREACHABLE;                      
                }

                // CRC von Kopfdaten pruefen.
                if (GetCRC16(status.get(), 6) != *(unsigned short*)(status.get() + 6)) {

                    trace("%s(%i):ERROR(1): %s [ head crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, GetCRC16(status.get(), 6), *(unsigned short*)(status.get() + 6));
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_CRC_HEAD;
                }
                // Laengenabgabe pruefen.
                if (*(unsigned int*)status.get() != (unsigned)(statusSize - 8)) {

                    trace("%s(%i):ERROR(1): %s [ length 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, *(unsigned int*)status.get(), (statusSize - 8));
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_RECEIVE;
                }
                // Gesamt-CRC pruefen
                if (GetCRC16(status.get() + 8, statusSize - 10) != *(unsigned short*)(status.get() + statusSize - 2)) {

                    trace("%s(%i):ERROR(1): %s [ paket crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, GetCRC16(status.get() + 8, statusSize - 10), *(unsigned short*)(status.get() + statusSize - 2));
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_CRC_PACKET;
                }
                unsigned char *lBuf = status.get() + 8;
                std::string _status(reinterpret_cast<char*>(lBuf), statusSize - 10);
                // Verschluesselung aktiv?
                if (isCrypt()) {

                    // Datenteil entschluesseln.
                    if ((res = readStreamCrypt(_status)) != IV_ERROR_SUCCESS) {

                        trace("%s(%i):ERROR(1): %s [ decrypt fail ]\n", __FILE__, __LINE__, __func__);
                        // Ggf. Worker im Active-Mode wieder starten.
                        resume();
                        return res;
                    }
                }
                // Daten im Puffer neu ausrichten.
                lBuf = status.get();
                for (i = 0; i < (int)_status.size() && i < (statusSize - 10); i++) {
                    lBuf[i] = (unsigned char)_status[i];
                }

                unsigned int s, c;
                s = *(unsigned int*)status.get();
                if (s <= (unsigned int)count) {
                    c = *(unsigned int*)&status[sizeof(unsigned int)];
                    crcChunk = s - (crcPos - const_cast<char*>(buf));
                    crc = GetCRC32(reinterpret_cast<unsigned char*>(crcPos), crcChunk, crc, s == (unsigned int)count);
                    crcPos += crcChunk;
                    trace("%s(%i):INFO(1): %s [ %3u: progression: %u / %u ( %u )\tcrc check: %s ]\n", __FILE__, __LINE__, __func__, statusCount++, s, count, count - s, (crc == c) ? "OK" : "FAIL");

                    if (crc != c) {

                        trace("%s(%i):ERROR(1): %s [ status progression crc 0x%04X != 0x%04X ]\n", __FILE__, __LINE__, __func__, c, crc);
                        // Ggf. Worker im Active-Mode wieder starten.
                        resume();
                        return -IV_ERROR_CRC_SEGMENT;
                    }
                } else {

                    trace("%s(%i):ERROR(1): %s [ status progression length %u > %u ]\n", __FILE__, __LINE__, __func__, s, count);
                    // Ggf. Worker im Active-Mode wieder starten.
                    resume();
                    return -IV_ERROR_CRC_SEGMENT;
                }

                // remoteCall-Back Funktion aufrufen, ggf. abbruch.
                if (_callBackFunction) 
                {
                    if (!_callBackFunction()) 
                    {	
                        trace("%s(%i):ERROR(1): %s [ CallBack abbruch ]\n", __FILE__, __LINE__, __func__);
                        // Reservierten Speicher freigeben.
                        // Ggf. Worker im Active-Mode wieder starten.
                        resume();
                        return -IV_ERROR_BREAK;
                    }
                }

                if (_callBackFunctionWithParams)
                {
                    if (!_callBackFunctionWithParams(count, s))
                    {					
                        _result = -IV_ERROR_BREAK;
                        // Fehlernummern setzen
                        _errorCommunicationNum = IV_ERROR_BREAK;
                        // Fehlertext ermitteln und eintragen.
                        getErrorCommunication(_errorCommunicationNum, _errorCommunication);

                        trace("%s(%i):ERROR(1): %s ( %04X )[%s]\n", __FILE__, __LINE__, __func__, -_result & 0xFFFF, _errorCommunication.c_str());
                    }
                }

            } while (*(int*)status.get() < count && res == IV_ERROR_SUCCESS);
        } else {

            // Zu wendige Antwortdaten
            _result = -IV_ERROR_RECEIVE;
        }

        // Ggf. Worker im Active-Mode wieder starten.
        resume();
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::readStream(short deviceId, streamType type, char *buf, int count)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X, %i ]\n", __FILE__, __LINE__, __func__, deviceId, type, buf, count);
    /*
    int addressOffset;
    int i, sizeData;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
    // Umgebung zuruecksetzen
    reset();

    // Remoteadresse setzen.
    setDeviceIdRemote(deviceId);

    // Gesamten Datenblock aufteilen
    for (addressOffset=0; addressOffset < count; addressOffset += IV_FLASH_SIZE_DATA)
    {
    // Datengroesze 
    sizeData = ((addressOffset + IV_FLASH_SIZE_DATA) > count) ? count - addressOffset : IV_FLASH_SIZE_DATA;

    // Meberpuffer aufbereiten
    *(unsigned int*)_packetData = address + addressOffset;
    // Angeforderte Laenge eintragen
    *(unsigned short*)&_packetData[sizeof(unsigned int)] = sizeData;

    // Variable aufbereiten
    sizeData = sizeof(unsigned int) + sizeof(unsigned short);

    if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_RECEIVE_STREAM, _packetData, sizeData)) != IV_ERROR_SUCCESS)
    {
    // Fehler ausgeben
    traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_FLASH);
    return _result;
    }

    // OK

    // Empfangene Daten uebernehmen
    for (i=0; i < (sizeData - (signed)sizeof(unsigned int)); i++)
    {				
    buf[addressOffset + i] = _packetData[sizeof(unsigned int) + i];
    }	
    }
    }
    else
    {
    unsigned char tmp[4];
    count = 0;

    // Antwort des Befehls senden.
    if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_RECEIVE_STREAM, tmp, count)) != IV_ERROR_SUCCESS)
    {			
    Trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
    return _result;
    }
    }
    */
    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::systemKeyCheck(short deviceId, char *key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, key);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Puffer loeschen
        memset(_packetData, 0, sizeof(_packetData));
        // Der Passwortpuffer wird mit 17 Zeichen verschluesselt
        count = 17;

        // Initialisieren der Verschluesselung
        df4LoadCryptKey();
        df4Init(TRUE);

        // Nur wenn Passwort angegeben wurde.
        if (strlen(key)) {

            // Passwort uebernehmen
            strncpy((char*)_packetData, key, 16);
        }

        // Verschluesseln
        df4(_packetData, count);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK);

            // Fehler auswerten
            if (((_result >> 8) & 0xFF) == 2)
            {
                // Funktion unbekannt
            }
            else if ((_result & 0xFF) == 1)
            {
                // Funktionsfehler, passwort falsch
            }

            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY_CHECK, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::systemKey(short deviceId, char *key)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceId, key);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Keine Daten zu senden
        count = 0;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY);
            return _result;
        }

        // n Antwortdaten erwartet
        if (count != 17)
        {
            trace("%s(%i):WARNING(1): %s [ Laenge des Passwortschlüssels mit %i Byte ist fragwuerdig ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert uebernehmen
        strcpy(key, (char*)_packetData);	
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SYSTEM_KEY, tmp, count)) != IV_ERROR_SUCCESS)
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::startTransparencyMode( unsigned char deviceId, unsigned short id, unsigned short flags, unsigned int sizeReadBuffer, unsigned int sizeWriteBuffer, int timeout )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %p, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, id, flags, sizeReadBuffer, sizeWriteBuffer, timeout);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        *reinterpret_cast<unsigned short *>( _packetData ) = id;
        count = sizeof(unsigned short);
        *reinterpret_cast<unsigned short *>( &_packetData[count] ) = flags;
        count += sizeof(unsigned short);
        *reinterpret_cast<unsigned int *>( &_packetData[count] ) = sizeReadBuffer;
        count += sizeof(unsigned int);
        *reinterpret_cast<unsigned int *>( &_packetData[count] ) = sizeWriteBuffer;
        count += sizeof(unsigned int);
        *reinterpret_cast<int *>( &_packetData[count] ) = timeout;
        count += sizeof(int);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE);
            return _result;
        }

        // Den nun genutzten Mode halten.
        if ( (flags & 0x01) == 0x01 )
            _transparencyMode = Raw;
        else
            _transparencyMode = Masked;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::stopTransparencyMode( unsigned char deviceId )
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceId );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        char exitCode[] = { '~', '~', '~', '\0' };
        if ((_result = Channel4Base::writeData( exitCode, sizeof(exitCode), count )) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_TRANSPARENCY_MODE);
            return _result;
        }

        // Den nun genutzten Mode halten.
        _transparencyMode = Off;
    }
    else
    {
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
bool Channel4ModuleSystem::isTransparencyMode()
{
    return _transparencyMode != Off;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::setLEDState( short deviceId, unsigned char slot, unsigned char id, unsigned char state )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, slot, id, state);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Den anzusprechenden Slot uebernehmen.
        _packetData[count] = slot;
        count += sizeof(unsigned char);
        // Die anzusprechende LED uebernehmen.
        _packetData[count] = id;
        count += sizeof(unsigned char);
        // Den gewünschten Zustand uebernehmen.
        _packetData[count] = state;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count)
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten.  ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_SET_LED, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleSystem::getLEDState(short deviceId, unsigned char slot, unsigned char id, unsigned char &state)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceId, slot, id, state);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceId)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceId);

        // Den anzusprechenden Slot uebernehmen.
        _packetData[count] = slot;
        count += sizeof(unsigned char);
        // Die anzusprechende LED uebernehmen.
        _packetData[count] = id;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceId, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED);
            return _result;
        }

        // OK
        if (count != 3 * sizeof(unsigned char))
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        state = (_packetData[2] != 0xFF) ? _packetData[2] : 0;
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_SYSTEM, IV_FUNCTION_GET_LED, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

#if !defined( DATAFOX_STUDIO_IV ) && !defined( DF_API_PRODUCTION )
// Funktions-Stumps

// MODULE_HARDWARE
int Channel4ModuleSystem::setTypeDisplay( short deviceId, unsigned char val, unsigned short value2, int checked )
{ 
    (void)val;
    (void)value2;
    (void)checked;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Channel4ModuleSystem::setTypeKeypad( short deviceId, unsigned char val, unsigned short value2, int checked )
{
    (void)val;
    (void)value2;
    (void)checked;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Channel4ModuleSystem::setTypeTransponder( short deviceId, unsigned char val, unsigned short value2, int checked )
{
    (void)val;
    (void)value2;
    (void)checked;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Channel4ModuleSystem::setSerial( short deviceId, unsigned int serial, unsigned int value2 )
{
    (void)serial;
    (void)value2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}
#endif
